# Copyright 2019 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of responsiveness to signals."""

import logging
import os
import signal
import subprocess
import sys
import tempfile
import threading
import unittest

import grpc

from tests.unit import _signal_client
from tests.unit import test_common

_CLIENT_PATH = None
if sys.executable is not None:
    _CLIENT_PATH = os.path.abspath(os.path.realpath(_signal_client.__file__))
else:
    # NOTE(rbellevi): For compatibility with internal testing.
    if len(sys.argv) != 2:
        raise RuntimeError("Must supply path to executable client.")
    client_name = sys.argv[1].split("/")[-1]
    del sys.argv[1]  # For compatibility with test runner.
    _CLIENT_PATH = os.path.realpath(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), client_name)
    )

_HOST = "localhost"

# The gevent test harness cannot run the monkeypatch code for the child process,
# so we need to instrument it manually.
_GEVENT_ARG = ("--gevent",) if test_common.running_under_gevent() else ()


class _GenericHandler:
    def __init__(self):
        self._connected_clients_lock = threading.RLock()
        self._connected_clients_event = threading.Event()
        self._connected_clients = 0

        self._unary_unary_handler = grpc.unary_unary_rpc_method_handler(
            self._handle_unary_unary
        )
        self._unary_stream_handler = grpc.unary_stream_rpc_method_handler(
            self._handle_unary_stream
        )

    def _on_client_connect(self):
        with self._connected_clients_lock:
            self._connected_clients += 1
            self._connected_clients_event.set()

    def _on_client_disconnect(self):
        with self._connected_clients_lock:
            self._connected_clients -= 1
            if self._connected_clients == 0:
                self._connected_clients_event.clear()

    def await_connected_client(self):
        """Blocks until a client connects to the server."""
        self._connected_clients_event.wait()

    def _handle_unary_unary(self, request, servicer_context):
        """Handles a unary RPC.

        Blocks until the client disconnects and then echoes.
        """
        stop_event = threading.Event()

        def on_rpc_end():
            self._on_client_disconnect()
            stop_event.set()

        servicer_context.add_callback(on_rpc_end)
        self._on_client_connect()
        stop_event.wait()
        return request

    def _handle_unary_stream(self, request, servicer_context):
        """Handles a server streaming RPC.

        Blocks until the client disconnects and then echoes.
        """
        stop_event = threading.Event()

        def on_rpc_end():
            self._on_client_disconnect()
            stop_event.set()

        servicer_context.add_callback(on_rpc_end)
        self._on_client_connect()
        stop_event.wait()
        yield request


def get_method_handlers(handler):
    return {
        _signal_client.UNARY_UNARY: handler._unary_unary_handler,
        _signal_client.UNARY_STREAM: handler._unary_stream_handler,
    }


def _read_stream(stream):
    stream.seek(0)
    return stream.read()


def _start_client(args, stdout, stderr):
    invocation = None
    if sys.executable is not None:
        invocation = (sys.executable, _CLIENT_PATH) + tuple(args)
    else:
        invocation = (_CLIENT_PATH,) + tuple(args)
    return subprocess.Popen(invocation, stdout=stdout, stderr=stderr)


class SignalHandlingTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._port = self._server.add_insecure_port("{}:0".format(_HOST))
        self._handler = _GenericHandler()
        self._server.add_registered_method_handlers(
            _signal_client._SERVICE_NAME, get_method_handlers(self._handler)
        )
        self._server.start()

    def tearDown(self):
        self._server.stop(None)

    @unittest.skipIf(os.name == "nt", "SIGINT not supported on windows")
    def testUnary(self):
        """Tests that the server unary code path does not stall signal handlers."""
        server_target = "{}:{}".format(_HOST, self._port)
        with tempfile.TemporaryFile(mode="r") as client_stdout:
            with tempfile.TemporaryFile(mode="r") as client_stderr:
                client = _start_client(
                    (server_target, "unary") + _GEVENT_ARG,
                    client_stdout,
                    client_stderr,
                )
                self._handler.await_connected_client()
                client.send_signal(signal.SIGINT)
                self.assertFalse(client.wait(), msg=_read_stream(client_stderr))
                client_stdout.seek(0)
                self.assertIn(
                    _signal_client.SIGTERM_MESSAGE, client_stdout.read()
                )

    @unittest.skipIf(os.name == "nt", "SIGINT not supported on windows")
    def testStreaming(self):
        """Tests that the server streaming code path does not stall signal handlers."""
        server_target = "{}:{}".format(_HOST, self._port)
        with tempfile.TemporaryFile(mode="r") as client_stdout:
            with tempfile.TemporaryFile(mode="r") as client_stderr:
                client = _start_client(
                    (server_target, "streaming") + _GEVENT_ARG,
                    client_stdout,
                    client_stderr,
                )
                self._handler.await_connected_client()
                client.send_signal(signal.SIGINT)
                self.assertFalse(client.wait(), msg=_read_stream(client_stderr))
                client_stdout.seek(0)
                self.assertIn(
                    _signal_client.SIGTERM_MESSAGE, client_stdout.read()
                )

    @unittest.skipIf(os.name == "nt", "SIGINT not supported on windows")
    def testUnaryWithException(self):
        server_target = "{}:{}".format(_HOST, self._port)
        with tempfile.TemporaryFile(mode="r") as client_stdout:
            with tempfile.TemporaryFile(mode="r") as client_stderr:
                client = _start_client(
                    ("--exception", server_target, "unary") + _GEVENT_ARG,
                    client_stdout,
                    client_stderr,
                )
                self._handler.await_connected_client()
                client.send_signal(signal.SIGINT)
                client.wait()
                self.assertEqual(0, client.returncode)

    @unittest.skipIf(os.name == "nt", "SIGINT not supported on windows")
    def testStreamingHandlerWithException(self):
        server_target = "{}:{}".format(_HOST, self._port)
        with tempfile.TemporaryFile(mode="r") as client_stdout:
            with tempfile.TemporaryFile(mode="r") as client_stderr:
                client = _start_client(
                    ("--exception", server_target, "streaming") + _GEVENT_ARG,
                    client_stdout,
                    client_stderr,
                )
                self._handler.await_connected_client()
                client.send_signal(signal.SIGINT)
                client.wait()
                client_stderr_output = _read_stream(client_stderr)
                try:
                    self.assertEqual(0, client.returncode)
                except AssertionError:
                    print(client_stderr_output, file=sys.stderr)
                    raise


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/compression/zstd/decompressor/v3/zstd.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/compression/zstd/decompressor/v3/zstd.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<envoy/extensions/compression/zstd/decompressor/v3/zstd.proto\x12\x31\x65nvoy.extensions.compression.zstd.decompressor.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"~\n\x04Zstd\x12\x36\n\x0c\x64ictionaries\x18\x01 \x03(\x0b\x32 .envoy.config.core.v3.DataSource\x12>\n\nchunk_size\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0c\xfa\x42\t*\x07\x18\x80\x80\x04(\x80 B\xbf\x01\n?io.envoyproxy.envoy.extensions.compression.zstd.decompressor.v3B\tZstdProtoP\x01Zggithub.com/envoyproxy/go-control-plane/envoy/extensions/compression/zstd/decompressor/v3;decompressorv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.compression.zstd.decompressor.v3.zstd_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n?io.envoyproxy.envoy.extensions.compression.zstd.decompressor.v3B\tZstdProtoP\001Zggithub.com/envoyproxy/go-control-plane/envoy/extensions/compression/zstd/decompressor/v3;decompressorv3\272\200\310\321\006\002\020\002'
  _globals['_ZSTD'].fields_by_name['chunk_size']._loaded_options = None
  _globals['_ZSTD'].fields_by_name['chunk_size']._serialized_options = b'\372B\t*\007\030\200\200\004(\200 '
  _globals['_ZSTD']._serialized_start=236
  _globals['_ZSTD']._serialized_end=362
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

cdef class ConnectivityEvent(BaseEvent):

  def __cinit__(
      self, grpc_completion_type completion_type, bint success, object tag):
    self.completion_type = completion_type
    self.success = success
    self.tag = tag


cdef class RequestCallEvent(BaseEvent):

  def __cinit__(
      self, grpc_completion_type completion_type, bint success, object tag,
      Call call, CallDetails call_details, tuple invocation_metadata):
    self.completion_type = completion_type
    self.success = success
    self.tag = tag
    self.call = call
    self.call_details = call_details
    self.invocation_metadata = invocation_metadata


cdef class BatchOperationEvent(BaseEvent):

  def __cinit__(
      self, grpc_completion_type completion_type, bint success, object tag,
      object batch_operations):
    self.completion_type = completion_type
    self.success = success
    self.tag = tag
    self.batch_operations = batch_operations


cdef class ServerShutdownEvent(BaseEvent):

  def __cinit__(
      self, grpc_completion_type completion_type, bint success, object tag):
    self.completion_type = completion_type
    self.success = success
    self.tag = tag



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# This function will ascii encode unicode string inputs if necessary.
# In Python3, unicode strings are the default str type.
cdef bytes str_to_bytes(object s):
  if s is None or isinstance(s, bytes):
    return s
  elif isinstance(s, unicode):
    return s.encode('ascii')
  else:
    raise TypeError('Expected bytes, str, or unicode, not {}'.format(type(s)))


# TODO(https://github.com/grpc/grpc/issues/13782): It would be nice for us if
# the type of metadata that we accept were exactly the same as the type of
# metadata that we deliver to our users (so "str" for this function's
# parameter rather than "object"), but would it be nice for our users? Right
# now we haven't yet heard from enough users to know one way or another.
cdef bytes _encode(object string_or_none):
  if string_or_none is None:
    return b''
  elif isinstance(string_or_none, (bytes,)):
    return <bytes>string_or_none
  elif isinstance(string_or_none, (unicode,)):
    return string_or_none.encode('utf8')
  else:
    raise TypeError('Expected str, not {}'.format(type(string_or_none)))


cdef str _decode(bytes bytestring):
    if isinstance(bytestring, (str,)):
        return <str>bytestring
    else:
        try:
            return bytestring.decode('utf8')
        except UnicodeDecodeError:
            _LOGGER.exception('Invalid encoding on %s', bytestring)
            return bytestring.decode('latin1')



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests server context abort mechanism"""

import collections
import gc
import logging
import unittest
import weakref

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_SERVICE_NAME = "test"
_ABORT = "abort"
_ABORT_WITH_STATUS = "AbortWithStatus"
_INVALID_CODE = "InvalidCode"

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"

_ABORT_DETAILS = "Abandon ship!"
_ABORT_METADATA = (("a-trailing-metadata", "42"),)


class _Status(
    collections.namedtuple("_Status", ("code", "details", "trailing_metadata")),
    grpc.Status,
):
    pass


class _Object:
    pass


do_not_leak_me = _Object()


def abort_unary_unary(request, servicer_context):
    this_should_not_be_leaked = do_not_leak_me
    servicer_context.abort(
        grpc.StatusCode.INTERNAL,
        _ABORT_DETAILS,
    )
    raise Exception("This line should not be executed!")


def abort_with_status_unary_unary(request, servicer_context):
    servicer_context.abort_with_status(
        _Status(
            code=grpc.StatusCode.INTERNAL,
            details=_ABORT_DETAILS,
            trailing_metadata=_ABORT_METADATA,
        )
    )
    raise Exception("This line should not be executed!")


def invalid_code_unary_unary(request, servicer_context):
    servicer_context.abort(
        42,
        _ABORT_DETAILS,
    )


RPC_METHOD_HANDLERS = {
    _ABORT: grpc.unary_unary_rpc_method_handler(abort_unary_unary),
    _ABORT_WITH_STATUS: grpc.unary_unary_rpc_method_handler(
        abort_with_status_unary_unary
    ),
    _INVALID_CODE: grpc.stream_stream_rpc_method_handler(
        invalid_code_unary_unary
    ),
}


class AbortTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        port = self._server.add_insecure_port("[::]:0")
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, RPC_METHOD_HANDLERS
        )
        self._server.start()

        self._channel = grpc.insecure_channel("localhost:%d" % port)

    def tearDown(self):
        self._channel.close()
        self._server.stop(0)

    def test_abort(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                grpc._common.fully_qualified_method(_SERVICE_NAME, _ABORT),
                _registered_method=True,
            )(_REQUEST)
        rpc_error = exception_context.exception

        self.assertEqual(rpc_error.code(), grpc.StatusCode.INTERNAL)
        self.assertEqual(rpc_error.details(), _ABORT_DETAILS)

    # This test ensures that abort() does not store the raised exception, which
    # on Python 3 (via the `__traceback__` attribute) holds a reference to
    # all local vars. Storing the raised exception can prevent GC and stop the
    # grpc_call from being unref'ed, even after server shutdown.
    @unittest.skip("https://github.com/grpc/grpc/issues/17927")
    def test_abort_does_not_leak_local_vars(self):
        global do_not_leak_me  # pylint: disable=global-statement
        weak_ref = weakref.ref(do_not_leak_me)

        # Servicer will abort() after creating a local ref to do_not_leak_me.
        with self.assertRaises(grpc.RpcError):
            self._channel.unary_unary(
                grpc._common.fully_qualified_method(_SERVICE_NAME, _ABORT),
                _registered_method=True,
            )(_REQUEST)

        # Server may still have a stack frame reference to the exception even
        # after client sees error, so ensure server has shutdown.
        self._server.stop(None)
        do_not_leak_me = None
        self.assertIsNone(weak_ref())

    def test_abort_with_status(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                grpc._common.fully_qualified_method(
                    _SERVICE_NAME, _ABORT_WITH_STATUS
                ),
                _registered_method=True,
            )(_REQUEST)
        rpc_error = exception_context.exception

        self.assertEqual(rpc_error.code(), grpc.StatusCode.INTERNAL)
        self.assertEqual(rpc_error.details(), _ABORT_DETAILS)
        self.assertEqual(rpc_error.trailing_metadata(), _ABORT_METADATA)

    def test_invalid_code(self):
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                grpc._common.fully_qualified_method(
                    _SERVICE_NAME, _INVALID_CODE
                ),
                _registered_method=True,
            )(_REQUEST)
        rpc_error = exception_context.exception

        self.assertEqual(rpc_error.code(), grpc.StatusCode.UNKNOWN)
        self.assertEqual(rpc_error.details(), _ABORT_DETAILS)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Testing the server context ability to access peer info."""

import asyncio
import logging
import os
from typing import Callable, Iterable, Sequence, Tuple
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework.common import test_constants
from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import TestServiceServicer
from tests_aio.unit._test_server import start_test_server

_REQUEST = b"\x03\x07"
_TEST_METHOD = "/test/UnaryUnary"


class TestContextPeer(AioTestBase):
    async def test_peer(self):
        @grpc.unary_unary_rpc_method_handler
        async def check_peer_unary_unary(
            request: bytes, context: aio.ServicerContext
        ):
            self.assertEqual(_REQUEST, request)
            # The peer address could be ipv4 or ipv6
            self.assertIn("ip", context.peer())
            return request

        # Creates a server
        server = aio.server()
        handlers = grpc.method_handlers_generic_handler(
            "test", {"UnaryUnary": check_peer_unary_unary}
        )
        server.add_generic_rpc_handlers((handlers,))
        port = server.add_insecure_port("[::]:0")
        await server.start()

        # Creates a channel
        async with aio.insecure_channel("localhost:%d" % port) as channel:
            response = await channel.unary_unary(_TEST_METHOD)(_REQUEST)
            self.assertEqual(_REQUEST, response)

        await server.stop(None)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2020 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import collections
import concurrent.futures
import datetime
import logging
import signal
import threading
import time
from typing import (
    DefaultDict,
    Dict,
    FrozenSet,
    Iterable,
    List,
    Mapping,
    Sequence,
    Set,
    Tuple,
)

import grpc
from grpc import _typing as grpc_typing
import grpc_admin
from grpc_channelz.v1 import channelz
from grpc_csm_observability import CsmOpenTelemetryPlugin
from opentelemetry.exporter.prometheus import PrometheusMetricReader
from opentelemetry.sdk.metrics import MeterProvider
from prometheus_client import start_http_server

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2
from src.proto.grpc.testing import test_pb2_grpc
from src.python.grpcio_tests.tests.fork import native_debug

native_debug.install_failure_signal_handler()

logger = logging.getLogger()
console_handler = logging.StreamHandler()
formatter = logging.Formatter(fmt="%(asctime)s: %(levelname)-8s %(message)s")
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)

_SUPPORTED_METHODS = (
    "UnaryCall",
    "EmptyCall",
)

_METHOD_CAMEL_TO_CAPS_SNAKE = {
    "UnaryCall": "UNARY_CALL",
    "EmptyCall": "EMPTY_CALL",
}

_METHOD_STR_TO_ENUM = {
    "UnaryCall": messages_pb2.ClientConfigureRequest.UNARY_CALL,
    "EmptyCall": messages_pb2.ClientConfigureRequest.EMPTY_CALL,
}

_METHOD_ENUM_TO_STR = {v: k for k, v in _METHOD_STR_TO_ENUM.items()}

_PROMETHEUS_PORT = 9464

PerMethodMetadataType = Mapping[str, Sequence[Tuple[str, str]]]


# FutureFromCall is both a grpc.Call and grpc.Future
class FutureFromCallType(grpc.Call, grpc.Future):
    pass


_CONFIG_CHANGE_TIMEOUT = datetime.timedelta(milliseconds=500)


class _StatsWatcher:
    _start: int
    _end: int
    _rpcs_needed: int
    _rpcs_by_peer: DefaultDict[str, int]
    _rpcs_by_method: DefaultDict[str, DefaultDict[str, int]]
    _no_remote_peer: int
    _lock: threading.Lock
    _condition: threading.Condition
    _metadata_keys: FrozenSet[str]
    _include_all_metadata: bool
    _metadata_by_peer: DefaultDict[
        str, messages_pb2.LoadBalancerStatsResponse.MetadataByPeer
    ]

    def __init__(self, start: int, end: int, metadata_keys: Iterable[str]):
        self._start = start
        self._end = end
        self._rpcs_needed = end - start
        self._rpcs_by_peer = collections.defaultdict(int)
        self._rpcs_by_method = collections.defaultdict(
            lambda: collections.defaultdict(int)
        )
        self._condition = threading.Condition()
        self._no_remote_peer = 0
        self._metadata_keys = frozenset(
            self._sanitize_metadata_key(key) for key in metadata_keys
        )
        self._include_all_metadata = "*" in self._metadata_keys
        self._metadata_by_peer = collections.defaultdict(
            messages_pb2.LoadBalancerStatsResponse.MetadataByPeer
        )

    @classmethod
    def _sanitize_metadata_key(cls, metadata_key: str) -> str:
        return metadata_key.strip().lower()

    def _add_metadata(
        self,
        rpc_metadata: messages_pb2.LoadBalancerStatsResponse.RpcMetadata,
        metadata_to_add: grpc_typing.MetadataType,
        metadata_type: messages_pb2.LoadBalancerStatsResponse.MetadataType,
    ) -> None:
        for key, value in metadata_to_add:
            if (
                self._include_all_metadata
                or self._sanitize_metadata_key(key) in self._metadata_keys
            ):
                rpc_metadata.metadata.append(
                    messages_pb2.LoadBalancerStatsResponse.MetadataEntry(
                        key=key, value=value, type=metadata_type
                    )
                )

    def on_rpc_complete(
        self,
        request_id: int,
        peer: str,
        method: str,
        *,
        initial_metadata: grpc_typing.MetadataType,
        trailing_metadata: grpc_typing.MetadataType,
    ) -> None:
        """Records statistics for a single RPC."""
        if self._start <= request_id < self._end:
            with self._condition:
                if not peer:
                    self._no_remote_peer += 1
                else:
                    self._rpcs_by_peer[peer] += 1
                    self._rpcs_by_method[method][peer] += 1
                    if self._metadata_keys:
                        rpc_metadata = (
                            messages_pb2.LoadBalancerStatsResponse.RpcMetadata()
                        )
                        self._add_metadata(
                            rpc_metadata,
                            initial_metadata,
                            messages_pb2.LoadBalancerStatsResponse.MetadataType.INITIAL,
                        )
                        self._add_metadata(
                            rpc_metadata,
                            trailing_metadata,
                            messages_pb2.LoadBalancerStatsResponse.MetadataType.TRAILING,
                        )
                        self._metadata_by_peer[peer].rpc_metadata.append(
                            rpc_metadata
                        )
                self._rpcs_needed -= 1
                self._condition.notify()

    def await_rpc_stats_response(
        self, timeout_sec: int
    ) -> messages_pb2.LoadBalancerStatsResponse:
        """Blocks until a full response has been collected."""
        with self._condition:
            self._condition.wait_for(
                lambda: not self._rpcs_needed, timeout=float(timeout_sec)
            )
            response = messages_pb2.LoadBalancerStatsResponse()
            for peer, count in self._rpcs_by_peer.items():
                response.rpcs_by_peer[peer] = count
            for method, count_by_peer in self._rpcs_by_method.items():
                for peer, count in count_by_peer.items():
                    response.rpcs_by_method[method].rpcs_by_peer[peer] = count
            for peer, metadata_by_peer in self._metadata_by_peer.items():
                response.metadatas_by_peer[peer].CopyFrom(metadata_by_peer)
            response.num_failures = self._no_remote_peer + self._rpcs_needed
        return response


_global_lock = threading.Lock()
_stop_event = threading.Event()
_global_rpc_id: int = 0
_watchers: Set[_StatsWatcher] = set()
_global_server = None
_global_rpcs_started: Mapping[str, int] = collections.defaultdict(int)
_global_rpcs_succeeded: Mapping[str, int] = collections.defaultdict(int)
_global_rpcs_failed: Mapping[str, int] = collections.defaultdict(int)

# Mapping[method, Mapping[status_code, count]]
_global_rpc_statuses: Mapping[str, Mapping[int, int]] = collections.defaultdict(
    lambda: collections.defaultdict(int)
)


def _handle_sigint(sig, frame) -> None:
    logger.warning("Received SIGINT")
    _stop_event.set()
    _global_server.stop(None)


class _LoadBalancerStatsServicer(
    test_pb2_grpc.LoadBalancerStatsServiceServicer
):
    def __init__(self):
        super(_LoadBalancerStatsServicer).__init__()

    def GetClientStats(
        self,
        request: messages_pb2.LoadBalancerStatsRequest,
        context: grpc.ServicerContext,
    ) -> messages_pb2.LoadBalancerStatsResponse:
        logger.info("Received stats request.")
        start = None
        end = None
        watcher = None
        with _global_lock:
            start = _global_rpc_id + 1
            end = start + request.num_rpcs
            watcher = _StatsWatcher(start, end, request.metadata_keys)
            _watchers.add(watcher)
        response = watcher.await_rpc_stats_response(request.timeout_sec)
        with _global_lock:
            _watchers.remove(watcher)
        logger.info("Returning stats response: %s", response)
        return response

    def GetClientAccumulatedStats(
        self,
        request: messages_pb2.LoadBalancerAccumulatedStatsRequest,
        context: grpc.ServicerContext,
    ) -> messages_pb2.LoadBalancerAccumulatedStatsResponse:
        logger.info("Received cumulative stats request.")
        response = messages_pb2.LoadBalancerAccumulatedStatsResponse()
        with _global_lock:
            for method in _SUPPORTED_METHODS:
                caps_method = _METHOD_CAMEL_TO_CAPS_SNAKE[method]
                response.num_rpcs_started_by_method[caps_method] = (
                    _global_rpcs_started[method]
                )
                response.num_rpcs_succeeded_by_method[caps_method] = (
                    _global_rpcs_succeeded[method]
                )
                response.num_rpcs_failed_by_method[caps_method] = (
                    _global_rpcs_failed[method]
                )
                response.stats_per_method[caps_method].rpcs_started = (
                    _global_rpcs_started[method]
                )
                for code, count in _global_rpc_statuses[method].items():
                    response.stats_per_method[caps_method].result[code] = count
        logger.info("Returning cumulative stats response.")
        return response


def _start_rpc(
    method: str,
    metadata: Sequence[Tuple[str, str]],
    request_id: int,
    stub: test_pb2_grpc.TestServiceStub,
    timeout: float,
    futures: Mapping[int, Tuple[FutureFromCallType, str]],
    request_payload_size: int,
    response_payload_size: int,
) -> None:
    logger.debug(f"Sending {method} request to backend: {request_id}")
    if method == "UnaryCall":
        request = messages_pb2.SimpleRequest(
            response_type=messages_pb2.COMPRESSABLE,
            response_size=response_payload_size,
            payload=messages_pb2.Payload(body=b"0" * request_payload_size),
        )
        future = stub.UnaryCall.future(
            request, metadata=metadata, timeout=timeout
        )
    elif method == "EmptyCall":
        if request_payload_size > 0:
            logger.error(
                f"request_payload_size should not be set for EMPTY_CALL"
            )
        if response_payload_size > 0:
            logger.error(
                f"response_payload_size should not be set for EMPTY_CALL"
            )
        future = stub.EmptyCall.future(
            empty_pb2.Empty(), metadata=metadata, timeout=timeout
        )
    else:
        raise ValueError(f"Unrecognized method '{method}'.")
    futures[request_id] = (future, method)


def _on_rpc_done(
    rpc_id: int, future: FutureFromCallType, method: str, print_response: bool
) -> None:
    exception = future.exception()
    hostname = ""
    with _global_lock:
        _global_rpc_statuses[method][future.code().value[0]] += 1
    if exception is not None:
        with _global_lock:
            _global_rpcs_failed[method] += 1
        if exception.code() == grpc.StatusCode.DEADLINE_EXCEEDED:
            logger.error(f"RPC {rpc_id} timed out")
        else:
            logger.error(exception)
    else:
        response = future.result()
        hostname = None
        for metadatum in future.initial_metadata():
            if metadatum[0] == "hostname":
                hostname = metadatum[1]
                break
        else:
            hostname = response.hostname
        if future.code() == grpc.StatusCode.OK:
            with _global_lock:
                _global_rpcs_succeeded[method] += 1
        else:
            with _global_lock:
                _global_rpcs_failed[method] += 1
        if print_response:
            if future.code() == grpc.StatusCode.OK:
                logger.debug("Successful response.")
            else:
                logger.debug(f"RPC failed: {rpc_id}")
    with _global_lock:
        for watcher in _watchers:
            watcher.on_rpc_complete(
                rpc_id,
                hostname,
                method,
                initial_metadata=future.initial_metadata(),
                trailing_metadata=future.trailing_metadata(),
            )


def _remove_completed_rpcs(
    rpc_futures: Mapping[int, FutureFromCallType], print_response: bool
) -> None:
    logger.debug("Removing completed RPCs")
    done = []
    for future_id, (future, method) in rpc_futures.items():
        if future.done():
            _on_rpc_done(future_id, future, method, args.print_response)
            done.append(future_id)
    for rpc_id in done:
        del rpc_futures[rpc_id]


def _cancel_all_rpcs(futures: Mapping[int, Tuple[grpc.Future, str]]) -> None:
    logger.info("Cancelling all remaining RPCs")
    for future, _ in futures.values():
        future.cancel()


class _ChannelConfiguration:
    """Configuration for a single client channel.

    Instances of this class are meant to be dealt with as PODs. That is,
    data member should be accessed directly. This class is not thread-safe.
    When accessing any of its members, the lock member should be held.
    """

    def __init__(
        self,
        method: str,
        metadata: Sequence[Tuple[str, str]],
        qps: int,
        server: str,
        rpc_timeout_sec: int,
        print_response: bool,
        secure_mode: bool,
        request_payload_size: int,
        response_payload_size: int,
    ):
        # condition is signalled when a change is made to the config.
        self.condition = threading.Condition()

        self.method = method
        self.metadata = metadata
        self.qps = qps
        self.server = server
        self.rpc_timeout_sec = rpc_timeout_sec
        self.print_response = print_response
        self.secure_mode = secure_mode
        self.response_payload_size = response_payload_size
        self.request_payload_size = request_payload_size


def _run_single_channel(config: _ChannelConfiguration) -> None:
    global _global_rpc_id  # pylint: disable=global-statement
    with config.condition:
        server = config.server
    channel = None
    if config.secure_mode:
        fallback_creds = grpc.experimental.insecure_channel_credentials()
        channel_creds = grpc.xds_channel_credentials(fallback_creds)
        channel = grpc.secure_channel(server, channel_creds)
    else:
        channel = grpc.insecure_channel(server)
    with channel:
        stub = test_pb2_grpc.TestServiceStub(channel)
        futures: Dict[int, Tuple[FutureFromCallType, str]] = {}
        while not _stop_event.is_set():
            with config.condition:
                if config.qps == 0:
                    config.condition.wait(
                        timeout=_CONFIG_CHANGE_TIMEOUT.total_seconds()
                    )
                    continue
                else:
                    duration_per_query = 1.0 / float(config.qps)
                request_id = None
                with _global_lock:
                    request_id = _global_rpc_id
                    _global_rpc_id += 1
                    _global_rpcs_started[config.method] += 1
                start = time.time()
                end = start + duration_per_query
                _start_rpc(
                    config.method,
                    config.metadata,
                    request_id,
                    stub,
                    float(config.rpc_timeout_sec),
                    futures,
                    config.request_payload_size,
                    config.response_payload_size,
                )
                print_response = config.print_response
            _remove_completed_rpcs(futures, config.print_response)
            logger.debug(f"Currently {len(futures)} in-flight RPCs")
            now = time.time()
            while now < end:
                time.sleep(end - now)
                now = time.time()
        _cancel_all_rpcs(futures)


class _XdsUpdateClientConfigureServicer(
    test_pb2_grpc.XdsUpdateClientConfigureServiceServicer
):
    def __init__(
        self, per_method_configs: Mapping[str, _ChannelConfiguration], qps: int
    ):
        super(_XdsUpdateClientConfigureServicer).__init__()
        self._per_method_configs = per_method_configs
        self._qps = qps

    def Configure(
        self,
        request: messages_pb2.ClientConfigureRequest,
        context: grpc.ServicerContext,
    ) -> messages_pb2.ClientConfigureResponse:
        logger.info("Received Configure RPC: %s", request)
        method_strs = [_METHOD_ENUM_TO_STR[t] for t in request.types]
        for method in _SUPPORTED_METHODS:
            method_enum = _METHOD_STR_TO_ENUM[method]
            channel_config = self._per_method_configs[method]
            if method in method_strs:
                qps = self._qps
                metadata = (
                    (md.key, md.value)
                    for md in request.metadata
                    if md.type == method_enum
                )
                # For backward compatibility, do not change timeout when we
                # receive a default value timeout.
                if request.timeout_sec == 0:
                    timeout_sec = channel_config.rpc_timeout_sec
                else:
                    timeout_sec = request.timeout_sec
            else:
                qps = 0
                metadata = ()
                # Leave timeout unchanged for backward compatibility.
                timeout_sec = channel_config.rpc_timeout_sec
            with channel_config.condition:
                channel_config.qps = qps
                channel_config.metadata = list(metadata)
                channel_config.rpc_timeout_sec = timeout_sec
                channel_config.condition.notify_all()
        return messages_pb2.ClientConfigureResponse()


class _MethodHandle:
    """An object grouping together threads driving RPCs for a method."""

    _channel_threads: List[threading.Thread]

    def __init__(
        self, num_channels: int, channel_config: _ChannelConfiguration
    ):
        """Creates and starts a group of threads running the indicated method."""
        self._channel_threads = []
        for i in range(num_channels):
            thread = threading.Thread(
                target=_run_single_channel, args=(channel_config,)
            )
            thread.start()
            self._channel_threads.append(thread)

    def stop(self) -> None:
        """Joins all threads referenced by the handle."""
        for channel_thread in self._channel_threads:
            channel_thread.join()


def _run(
    args: argparse.Namespace,
    methods: Sequence[str],
    per_method_metadata: PerMethodMetadataType,
) -> None:
    logger.info("Starting python xDS Interop Client.")
    csm_plugin = None
    if args.enable_csm_observability:
        csm_plugin = _prepare_csm_observability_plugin()
        csm_plugin.register_global()
    global _global_server  # pylint: disable=global-statement
    method_handles = []
    channel_configs = {}
    for method in _SUPPORTED_METHODS:
        if method in methods:
            qps = args.qps
        else:
            qps = 0
        channel_config = _ChannelConfiguration(
            method,
            per_method_metadata.get(method, []),
            qps,
            args.server,
            args.rpc_timeout_sec,
            args.print_response,
            args.secure_mode,
            args.request_payload_size,
            args.response_payload_size,
        )
        channel_configs[method] = channel_config
        method_handles.append(_MethodHandle(args.num_channels, channel_config))
    _global_server = grpc.server(concurrent.futures.ThreadPoolExecutor())
    _global_server.add_insecure_port(f"0.0.0.0:{args.stats_port}")
    test_pb2_grpc.add_LoadBalancerStatsServiceServicer_to_server(
        _LoadBalancerStatsServicer(), _global_server
    )
    test_pb2_grpc.add_XdsUpdateClientConfigureServiceServicer_to_server(
        _XdsUpdateClientConfigureServicer(channel_configs, args.qps),
        _global_server,
    )
    grpc_admin.add_admin_servicers(_global_server)
    _global_server.start()
    _global_server.wait_for_termination()
    for method_handle in method_handles:
        method_handle.stop()
    if csm_plugin:
        csm_plugin.deregister_global()


def parse_metadata_arg(metadata_arg: str) -> PerMethodMetadataType:
    metadata = metadata_arg.split(",") if args.metadata else []
    per_method_metadata = collections.defaultdict(list)
    for metadatum in metadata:
        elems = metadatum.split(":")
        if len(elems) != 3:
            raise ValueError(
                f"'{metadatum}' was not in the form 'METHOD:KEY:VALUE'"
            )
        if elems[0] not in _SUPPORTED_METHODS:
            raise ValueError(f"Unrecognized method '{elems[0]}'")
        per_method_metadata[elems[0]].append((elems[1], elems[2]))
    return per_method_metadata


def parse_rpc_arg(rpc_arg: str) -> Sequence[str]:
    methods = rpc_arg.split(",")
    if set(methods) - set(_SUPPORTED_METHODS):
        raise ValueError(
            "--rpc supported methods: {}".format(", ".join(_SUPPORTED_METHODS))
        )
    return methods


def bool_arg(arg: str) -> bool:
    if arg.lower() in ("true", "yes", "y"):
        return True
    elif arg.lower() in ("false", "no", "n"):
        return False
    else:
        raise argparse.ArgumentTypeError(f"Could not parse '{arg}' as a bool.")


def _prepare_csm_observability_plugin() -> CsmOpenTelemetryPlugin:
    # Start Prometheus client
    start_http_server(port=_PROMETHEUS_PORT, addr="0.0.0.0")
    reader = PrometheusMetricReader()
    meter_provider = MeterProvider(metric_readers=[reader])
    csm_plugin = CsmOpenTelemetryPlugin(
        meter_provider=meter_provider,
    )
    return csm_plugin


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Run Python XDS interop client."
    )
    parser.add_argument(
        "--num_channels",
        default=1,
        type=int,
        help="The number of channels from which to send requests.",
    )
    parser.add_argument(
        "--print_response",
        default="False",
        type=bool_arg,
        help="Write RPC response to STDOUT.",
    )
    parser.add_argument(
        "--qps",
        default=1,
        type=int,
        help="The number of queries to send from each channel per second.",
    )
    parser.add_argument(
        "--rpc_timeout_sec",
        default=30,
        type=int,
        help="The per-RPC timeout in seconds.",
    )
    parser.add_argument(
        "--server", default="localhost:50051", help="The address of the server."
    )
    parser.add_argument(
        "--stats_port",
        default=50052,
        type=int,
        help="The port on which to expose the peer distribution stats service.",
    )
    parser.add_argument(
        "--secure_mode",
        default="False",
        type=bool_arg,
        help="If specified, uses xDS credentials to connect to the server.",
    )
    parser.add_argument(
        "--verbose",
        help="verbose log output",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "--log_file", default=None, type=str, help="A file to log to."
    )
    parser.add_argument(
        "--enable_csm_observability",
        help="Whether to enable CSM Observability",
        default="False",
        type=bool_arg,
    )
    parser.add_argument(
        "--request_payload_size",
        default=0,
        type=int,
        help="Set the SimpleRequest.payload.body to a string of repeated 0 (zero) ASCII characters of the given size in bytes.",
    )
    parser.add_argument(
        "--response_payload_size",
        default=0,
        type=int,
        help="Ask the server to respond with SimpleResponse.payload.body of the given length (may not be implemented on the server).",
    )
    rpc_help = "A comma-delimited list of RPC methods to run. Must be one of "
    rpc_help += ", ".join(_SUPPORTED_METHODS)
    rpc_help += "."
    parser.add_argument("--rpc", default="UnaryCall", type=str, help=rpc_help)
    metadata_help = (
        "A comma-delimited list of 3-tuples of the form "
        + "METHOD:KEY:VALUE, e.g. "
        + "EmptyCall:key1:value1,UnaryCall:key2:value2,EmptyCall:k3:v3"
    )
    parser.add_argument("--metadata", default="", type=str, help=metadata_help)
    args = parser.parse_args()
    signal.signal(signal.SIGINT, _handle_sigint)
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    if args.log_file:
        file_handler = logging.FileHandler(args.log_file, mode="a")
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
    _run(args, parse_rpc_arg(args.rpc), parse_metadata_arg(args.metadata))



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import csv
import glob
import math
import multiprocessing
import os
import pathlib
import shutil
import subprocess
import sys

sys.path.append(
    os.path.join(
        os.path.dirname(sys.argv[0]), "..", "..", "run_tests", "python_utils"
    )
)
import check_on_pr

argp = argparse.ArgumentParser(description="Perform diff on microbenchmarks")

argp.add_argument(
    "-d",
    "--diff_base",
    type=str,
    help="Commit or branch to compare the current one to",
)

argp.add_argument("-j", "--jobs", type=int, default=multiprocessing.cpu_count())

args = argp.parse_args()

# the libraries for which check bloat difference is calculated
LIBS = [
    "libgrpc.so",
    "libgrpc++.so",
]


def _build(output_dir):
    """Perform the cmake build under the output_dir."""
    shutil.rmtree(output_dir, ignore_errors=True)
    subprocess.check_call("mkdir -p %s" % output_dir, shell=True, cwd=".")
    subprocess.check_call(
        [
            "cmake",
            "-DgRPC_BUILD_TESTS=OFF",
            "-DCMAKE_CXX_STANDARD=17",
            "-DBUILD_SHARED_LIBS=ON",
            "-DCMAKE_BUILD_TYPE=RelWithDebInfo",
            '-DCMAKE_C_FLAGS="-gsplit-dwarf"',
            '-DCMAKE_CXX_FLAGS="-gsplit-dwarf"',
            "..",
        ],
        cwd=output_dir,
    )
    subprocess.check_call("make -j%d" % args.jobs, shell=True, cwd=output_dir)


def _rank_diff_bytes(diff_bytes):
    """Determine how significant diff_bytes is, and return a simple integer representing that"""
    mul = 1
    if diff_bytes < 0:
        mul = -1
        diff_bytes = -diff_bytes
    if diff_bytes < 2 * 1024:
        return 0
    if diff_bytes < 16 * 1024:
        return 1 * mul
    if diff_bytes < 128 * 1024:
        return 2 * mul
    return 3 * mul


_build("bloat_diff_new")

if args.diff_base:
    where_am_i = (
        subprocess.check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"])
        .decode()
        .strip()
    )
    # checkout the diff base (="old")
    subprocess.check_call(["git", "checkout", args.diff_base])
    subprocess.check_call(["git", "submodule", "update"])
    try:
        _build("bloat_diff_old")
    finally:
        # restore the original revision (="new")
        subprocess.check_call(["git", "checkout", where_am_i])
        subprocess.check_call(["git", "submodule", "update"])

pathlib.Path("bloaty-build").mkdir(exist_ok=True)
subprocess.check_call(
    ["cmake", "-G", "Unix Makefiles", "../third_party/bloaty"],
    cwd="bloaty-build",
)
subprocess.check_call(
    "make -j%d bloaty" % args.jobs, shell=True, cwd="bloaty-build"
)

text = ""
diff_size = 0
for lib in LIBS:
    text += (
        "****************************************************************\n\n"
    )
    text += lib + "\n\n"
    old_version = glob.glob("bloat_diff_old/%s" % lib)
    new_version = glob.glob("bloat_diff_new/%s" % lib)
    for filename in [old_version, new_version]:
        if filename:
            subprocess.check_call(
                "strip %s -o %s.stripped" % (filename[0], filename[0]),
                shell=True,
            )
    assert len(new_version) == 1
    cmd = "bloaty-build/bloaty -d compileunits,symbols"
    if old_version:
        assert len(old_version) == 1
        text += subprocess.check_output(
            "%s -n 0 --debug-file=%s --debug-file=%s %s.stripped -- %s.stripped"
            % (
                cmd,
                new_version[0],
                old_version[0],
                new_version[0],
                old_version[0],
            ),
            shell=True,
        ).decode()
        sections = [
            x
            for x in csv.reader(
                subprocess.check_output(
                    "bloaty-build/bloaty -n 0 --csv %s -- %s"
                    % (new_version[0], old_version[0]),
                    shell=True,
                )
                .decode()
                .splitlines()
            )
        ]
        print(sections)
        for section in sections[1:]:
            # skip debug sections for bloat severity calculation
            if section[0].startswith(".debug"):
                continue
            # skip dynamic loader sections too
            if section[0].startswith(".dyn"):
                continue
            diff_size += int(section[2])
    else:
        text += subprocess.check_output(
            "%s %s.stripped -n 0 --debug-file=%s"
            % (cmd, new_version[0], new_version[0]),
            shell=True,
        ).decode()
    text += "\n\n"

severity = _rank_diff_bytes(diff_size)
print("SEVERITY: %d" % severity)

print(text)
check_on_pr.check_on_pr("Bloat Difference", "```\n%s\n```" % text)
check_on_pr.label_significance_on_pr("bloat", severity)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/early_header_mutation/header_mutation/v3/header_mutation.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/early_header_mutation/header_mutation/v3/header_mutation.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.common.mutation_rules.v3 import mutation_rules_pb2 as envoy_dot_config_dot_common_dot_mutation__rules_dot_v3_dot_mutation__rules__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nTenvoy/extensions/http/early_header_mutation/header_mutation/v3/header_mutation.proto\x12>envoy.extensions.http.early_header_mutation.header_mutation.v3\x1a:envoy/config/common/mutation_rules/v3/mutation_rules.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"d\n\x0eHeaderMutation\x12R\n\tmutations\x18\x01 \x03(\x0b\x32\x35.envoy.config.common.mutation_rules.v3.HeaderMutationB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x42\xe6\x01\nLio.envoyproxy.envoy.extensions.http.early_header_mutation.header_mutation.v3B\x13HeaderMutationProtoP\x01Zwgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/early_header_mutation/header_mutation/v3;header_mutationv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.early_header_mutation.header_mutation.v3.header_mutation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nLio.envoyproxy.envoy.extensions.http.early_header_mutation.header_mutation.v3B\023HeaderMutationProtoP\001Zwgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/early_header_mutation/header_mutation/v3;header_mutationv3\272\200\310\321\006\002\020\002'
  _globals['_HEADERMUTATION'].fields_by_name['mutations']._loaded_options = None
  _globals['_HEADERMUTATION'].fields_by_name['mutations']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_HEADERMUTATION']._serialized_start=268
  _globals['_HEADERMUTATION']._serialized_end=368
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/api/v2/endpoint.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/api/v2/endpoint.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.endpoint import endpoint_components_pb2 as envoy_dot_api_dot_v2_dot_endpoint_dot_endpoint__components__pb2
from envoy.type import percent_pb2 as envoy_dot_type_dot_percent__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x65nvoy/api/v2/endpoint.proto\x12\x0c\x65nvoy.api.v2\x1a/envoy/api/v2/endpoint/endpoint_components.proto\x1a\x18\x65nvoy/type/percent.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd1\x05\n\x15\x43lusterLoadAssignment\x12\x1d\n\x0c\x63luster_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02 \x01\x12=\n\tendpoints\x18\x02 \x03(\x0b\x32*.envoy.api.v2.endpoint.LocalityLbEndpoints\x12P\n\x0fnamed_endpoints\x18\x05 \x03(\x0b\x32\x37.envoy.api.v2.ClusterLoadAssignment.NamedEndpointsEntry\x12:\n\x06policy\x18\x04 \x01(\x0b\x32*.envoy.api.v2.ClusterLoadAssignment.Policy\x1a\xf3\x02\n\x06Policy\x12O\n\x0e\x64rop_overloads\x18\x02 \x03(\x0b\x32\x37.envoy.api.v2.ClusterLoadAssignment.Policy.DropOverload\x12\x46\n\x17overprovisioning_factor\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02 \x00\x12\x41\n\x14\x65ndpoint_stale_after\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02*\x00\x12$\n\x18\x64isable_overprovisioning\x18\x05 \x01(\x08\x42\x02\x18\x01\x1a\x61\n\x0c\x44ropOverload\x12\x19\n\x08\x63\x61tegory\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02 \x01\x12\x36\n\x0f\x64rop_percentage\x18\x02 \x01(\x0b\x32\x1d.envoy.type.FractionalPercentJ\x04\x08\x01\x10\x02\x1aV\n\x13NamedEndpointsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12.\n\x05value\x18\x02 \x01(\x0b\x32\x1f.envoy.api.v2.endpoint.Endpoint:\x02\x38\x01\x42\x90\x01\n\x1aio.envoyproxy.envoy.api.v2B\rEndpointProtoP\x01Z9github.com/envoyproxy/go-control-plane/envoy/api/v2;apiv2\xf2\x98\xfe\x8f\x05\x1a\x12\x18\x65nvoy.config.endpoint.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.api.v2.endpoint_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\032io.envoyproxy.envoy.api.v2B\rEndpointProtoP\001Z9github.com/envoyproxy/go-control-plane/envoy/api/v2;apiv2\362\230\376\217\005\032\022\030envoy.config.endpoint.v3\272\200\310\321\006\002\020\001'
  _globals['_CLUSTERLOADASSIGNMENT_POLICY_DROPOVERLOAD'].fields_by_name['category']._loaded_options = None
  _globals['_CLUSTERLOADASSIGNMENT_POLICY_DROPOVERLOAD'].fields_by_name['category']._serialized_options = b'\372B\004r\002 \001'
  _globals['_CLUSTERLOADASSIGNMENT_POLICY'].fields_by_name['overprovisioning_factor']._loaded_options = None
  _globals['_CLUSTERLOADASSIGNMENT_POLICY'].fields_by_name['overprovisioning_factor']._serialized_options = b'\372B\004*\002 \000'
  _globals['_CLUSTERLOADASSIGNMENT_POLICY'].fields_by_name['endpoint_stale_after']._loaded_options = None
  _globals['_CLUSTERLOADASSIGNMENT_POLICY'].fields_by_name['endpoint_stale_after']._serialized_options = b'\372B\005\252\001\002*\000'
  _globals['_CLUSTERLOADASSIGNMENT_POLICY'].fields_by_name['disable_overprovisioning']._loaded_options = None
  _globals['_CLUSTERLOADASSIGNMENT_POLICY'].fields_by_name['disable_overprovisioning']._serialized_options = b'\030\001'
  _globals['_CLUSTERLOADASSIGNMENT_NAMEDENDPOINTSENTRY']._loaded_options = None
  _globals['_CLUSTERLOADASSIGNMENT_NAMEDENDPOINTSENTRY']._serialized_options = b'8\001'
  _globals['_CLUSTERLOADASSIGNMENT'].fields_by_name['cluster_name']._loaded_options = None
  _globals['_CLUSTERLOADASSIGNMENT'].fields_by_name['cluster_name']._serialized_options = b'\372B\004r\002 \001'
  _globals['_CLUSTERLOADASSIGNMENT']._serialized_start=273
  _globals['_CLUSTERLOADASSIGNMENT']._serialized_end=994
  _globals['_CLUSTERLOADASSIGNMENT_POLICY']._serialized_start=535
  _globals['_CLUSTERLOADASSIGNMENT_POLICY']._serialized_end=906
  _globals['_CLUSTERLOADASSIGNMENT_POLICY_DROPOVERLOAD']._serialized_start=803
  _globals['_CLUSTERLOADASSIGNMENT_POLICY_DROPOVERLOAD']._serialized_end=900
  _globals['_CLUSTERLOADASSIGNMENT_NAMEDENDPOINTSENTRY']._serialized_start=908
  _globals['_CLUSTERLOADASSIGNMENT_NAMEDENDPOINTSENTRY']._serialized_end=994
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

expected_files = [
    "include/grpc++/create_channel_posix.h",
    "include/grpc++/server_builder.h",
    "include/grpc++/resource_quota.h",
    "include/grpc++/create_channel.h",
    "include/grpc++/alarm.h",
    "include/grpc++/server.h",
    "include/grpc++/server_context.h",
    "include/grpc++/client_context.h",
    "include/grpc++/server_posix.h",
    "include/grpc++/grpc++.h",
    "include/grpc++/health_check_service_interface.h",
    "include/grpc++/completion_queue.h",
    "include/grpc++/channel.h",
    "include/grpc++/support/sync_stream.h",
    "include/grpc++/support/status.h",
    "include/grpc++/support/config.h",
    "include/grpc++/support/status_code_enum.h",
    "include/grpc++/support/byte_buffer.h",
    "include/grpc++/support/error_details.h",
    "include/grpc++/support/async_unary_call.h",
    "include/grpc++/support/channel_arguments.h",
    "include/grpc++/support/async_stream.h",
    "include/grpc++/support/slice.h",
    "include/grpc++/support/stub_options.h",
    "include/grpc++/support/string_ref.h",
    "include/grpc++/support/time.h",
    "include/grpc++/security/auth_metadata_processor.h",
    "include/grpc++/security/credentials.h",
    "include/grpc++/security/server_credentials.h",
    "include/grpc++/security/auth_context.h",
    "include/grpc++/impl/rpc_method.h",
    "include/grpc++/impl/server_builder_option.h",
    "include/grpc++/impl/call.h",
    "include/grpc++/impl/service_type.h",
    "include/grpc++/impl/grpc_library.h",
    "include/grpc++/impl/client_unary_call.h",
    "include/grpc++/impl/channel_argument_option.h",
    "include/grpc++/impl/rpc_service_method.h",
    "include/grpc++/impl/method_handler_impl.h",
    "include/grpc++/impl/server_builder_plugin.h",
    "include/grpc++/impl/server_initializer.h",
    "include/grpc++/impl/serialization_traits.h",
    "include/grpc++/impl/codegen/sync_stream.h",
    "include/grpc++/impl/codegen/channel_interface.h",
    "include/grpc++/impl/codegen/config_protobuf.h",
    "include/grpc++/impl/codegen/status.h",
    "include/grpc++/impl/codegen/config.h",
    "include/grpc++/impl/codegen/status_code_enum.h",
    "include/grpc++/impl/codegen/metadata_map.h",
    "include/grpc++/impl/codegen/rpc_method.h",
    "include/grpc++/impl/codegen/server_context.h",
    "include/grpc++/impl/codegen/byte_buffer.h",
    "include/grpc++/impl/codegen/async_unary_call.h",
    "include/grpc++/impl/codegen/server_interface.h",
    "include/grpc++/impl/codegen/call.h",
    "include/grpc++/impl/codegen/client_context.h",
    "include/grpc++/impl/codegen/service_type.h",
    "include/grpc++/impl/codegen/async_stream.h",
    "include/grpc++/impl/codegen/slice.h",
    "include/grpc++/impl/codegen/client_unary_call.h",
    "include/grpc++/impl/codegen/proto_utils.h",
    "include/grpc++/impl/codegen/stub_options.h",
    "include/grpc++/impl/codegen/rpc_service_method.h",
    "include/grpc++/impl/codegen/method_handler_impl.h",
    "include/grpc++/impl/codegen/string_ref.h",
    "include/grpc++/impl/codegen/completion_queue_tag.h",
    "include/grpc++/impl/codegen/completion_queue.h",
    "include/grpc++/impl/codegen/serialization_traits.h",
    "include/grpc++/impl/codegen/create_auth_context.h",
    "include/grpc++/impl/codegen/time.h",
    "include/grpc++/impl/codegen/security/auth_context.h",
    "include/grpc++/ext/health_check_service_server_builder_option.h",
    "include/grpc++/ext/proto_server_reflection_plugin.h",
    "include/grpc++/generic/async_generic_service.h",
    "include/grpc++/generic/generic_stub.h",
    "include/grpc++/test/mock_stream.h",
    "include/grpc++/test/server_context_test_spouse.h",
]

file_template = """//
//
// Copyright 2018 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//

// DEPRECATED: The headers in include/grpc++ are deprecated. Please include the
// headers in include/grpcpp instead. This header exists only for backwards
// compatibility.

#ifndef GRPCXX_FILE_PATH_NAME_UPPER
#define GRPCXX_FILE_PATH_NAME_UPPER

#include <grpcpp/FILE_PATH_NAME_LOWER>

#endif  // GRPCXX_FILE_PATH_NAME_UPPER
"""

errors = 0

path_files = []
for root, dirs, files in os.walk("include/grpc++"):
    for filename in files:
        path_file = os.path.join(root, filename)
        path_files.append(path_file)

if path_files.sort() != expected_files.sort():
    diff_plus = [file for file in path_files if file not in expected_files]
    diff_minus = [file for file in expected_files if file not in path_files]
    for file in diff_minus:
        print(("- ", file))
    for file in diff_plus:
        print(("+ ", file))
    errors += 1

if errors > 0:
    sys.exit(errors)

for path_file in expected_files:
    relative_path_file = path_file.split("/", 2)[2]

    replace_lower = relative_path_file.replace("+", "p")

    replace_upper = relative_path_file.replace("/", "_")
    replace_upper = replace_upper.replace(".", "_")
    replace_upper = replace_upper.upper().replace("+", "X")

    expected_content = file_template.replace(
        "FILE_PATH_NAME_LOWER", replace_lower
    )
    expected_content = expected_content.replace(
        "FILE_PATH_NAME_UPPER", replace_upper
    )

    path_file_expected = path_file + ".expected"
    with open(path_file_expected, "w") as fo:
        fo.write(expected_content)

    if 0 != os.system("diff %s %s" % (path_file_expected, path_file)):
        print(("Difference found in file:", path_file))
        errors += 1

    os.remove(path_file_expected)

check_extensions = [".h", ".cc", ".c", ".m"]

for root, dirs, files in os.walk("src"):
    for filename in files:
        path_file = os.path.join(root, filename)
        for ext in check_extensions:
            if path_file.endswith(ext):
                try:
                    with open(path_file, "r") as fi:
                        content = fi.read()
                        if "#include <grpc++/" in content:
                            print(
                                "Failed: invalid include of deprecated headers"
                                " in include/grpc++ in %s" % path_file
                            )
                            errors += 1
                except IOError:
                    pass

sys.exit(errors)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef class Operation:

  cdef void c(self) except *:
    raise NotImplementedError()

  cdef void un_c(self) except *:
    raise NotImplementedError()


cdef class SendInitialMetadataOperation(Operation):

  def __cinit__(self, initial_metadata, flags):
    self._initial_metadata = initial_metadata
    self._flags = flags

  def type(self):
    return GRPC_OP_SEND_INITIAL_METADATA

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_SEND_INITIAL_METADATA
    self.c_op.flags = self._flags
    _store_c_metadata(
        self._initial_metadata, &self._c_initial_metadata,
        &self._c_initial_metadata_count)
    self.c_op.data.send_initial_metadata.metadata = self._c_initial_metadata
    self.c_op.data.send_initial_metadata.count = self._c_initial_metadata_count
    self.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0

  cdef void un_c(self) except *:
    _release_c_metadata(
        self._c_initial_metadata, self._c_initial_metadata_count)


cdef class SendMessageOperation(Operation):

  def __cinit__(self, bytes message, int flags):
    if message is None:
      self._message = b''
    else:
      self._message = message
    self._flags = flags

  def type(self):
    return GRPC_OP_SEND_MESSAGE

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_SEND_MESSAGE
    self.c_op.flags = self._flags
    cdef grpc_slice message_slice = grpc_slice_from_copied_buffer(
        self._message, len(self._message))
    self._c_message_byte_buffer = grpc_raw_byte_buffer_create(
        &message_slice, 1)
    grpc_slice_unref(message_slice)
    self.c_op.data.send_message.send_message = self._c_message_byte_buffer

  cdef void un_c(self) except *:
    grpc_byte_buffer_destroy(self._c_message_byte_buffer)


cdef class SendCloseFromClientOperation(Operation):

  def __cinit__(self, int flags):
    self._flags = flags

  def type(self):
    return GRPC_OP_SEND_CLOSE_FROM_CLIENT

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_SEND_CLOSE_FROM_CLIENT
    self.c_op.flags = self._flags

  cdef void un_c(self) except *:
    pass


cdef class SendStatusFromServerOperation(Operation):

  def __cinit__(self, trailing_metadata, code, object details, int flags):
    self._trailing_metadata = trailing_metadata
    self._code = code
    self._details = details
    self._flags = flags

  def type(self):
    return GRPC_OP_SEND_STATUS_FROM_SERVER

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_SEND_STATUS_FROM_SERVER
    self.c_op.flags = self._flags
    _store_c_metadata(
        self._trailing_metadata, &self._c_trailing_metadata,
        &self._c_trailing_metadata_count)
    self.c_op.data.send_status_from_server.trailing_metadata = (
        self._c_trailing_metadata)
    self.c_op.data.send_status_from_server.trailing_metadata_count = (
        self._c_trailing_metadata_count)
    self.c_op.data.send_status_from_server.status = self._code
    self._c_details = _slice_from_bytes(_encode(self._details))
    self.c_op.data.send_status_from_server.status_details = &self._c_details

  cdef void un_c(self) except *:
    grpc_slice_unref(self._c_details)
    _release_c_metadata(
        self._c_trailing_metadata, self._c_trailing_metadata_count)


cdef class ReceiveInitialMetadataOperation(Operation):

  def __cinit__(self, flags):
    self._flags = flags

  def type(self):
    return GRPC_OP_RECV_INITIAL_METADATA

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_RECV_INITIAL_METADATA
    self.c_op.flags = self._flags
    grpc_metadata_array_init(&self._c_initial_metadata)
    self.c_op.data.receive_initial_metadata.receive_initial_metadata = (
        &self._c_initial_metadata)

  cdef void un_c(self) except *:
    self._initial_metadata = _metadata(&self._c_initial_metadata)
    grpc_metadata_array_destroy(&self._c_initial_metadata)

  def initial_metadata(self):
    return self._initial_metadata


cdef class ReceiveMessageOperation(Operation):

  def __cinit__(self, flags):
    self._flags = flags

  def type(self):
    return GRPC_OP_RECV_MESSAGE

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_RECV_MESSAGE
    self.c_op.flags = self._flags
    self.c_op.data.receive_message.receive_message = (
        &self._c_message_byte_buffer)

  cdef void un_c(self) except *:
    cdef grpc_byte_buffer_reader message_reader
    cdef bint message_reader_status
    cdef grpc_slice message_slice
    cdef size_t message_slice_length
    cdef list chunks = []

    if self._c_message_byte_buffer != NULL:
      message_reader_status = grpc_byte_buffer_reader_init(
          &message_reader, self._c_message_byte_buffer)
      if message_reader_status:
        while grpc_byte_buffer_reader_next(&message_reader, &message_slice):
          message_slice_length = grpc_slice_length(message_slice)
          if message_slice_length > 0:
            chunks.append((<char *>grpc_slice_start_ptr(message_slice))[:message_slice_length])
          grpc_slice_unref(message_slice)

        grpc_byte_buffer_reader_destroy(&message_reader)
        self._message = b"".join(chunks)
      else:
        self._message = None
      grpc_byte_buffer_destroy(self._c_message_byte_buffer)
    else:
      self._message = None

  def message(self):
    return self._message


cdef class ReceiveStatusOnClientOperation(Operation):

  def __cinit__(self, flags):
    self._flags = flags

  def type(self):
    return GRPC_OP_RECV_STATUS_ON_CLIENT

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_RECV_STATUS_ON_CLIENT
    self.c_op.flags = self._flags
    grpc_metadata_array_init(&self._c_trailing_metadata)
    self.c_op.data.receive_status_on_client.trailing_metadata = (
        &self._c_trailing_metadata)
    self.c_op.data.receive_status_on_client.status = (
        &self._c_code)
    self.c_op.data.receive_status_on_client.status_details = (
        &self._c_details)
    self.c_op.data.receive_status_on_client.error_string = (
        &self._c_error_string)

  cdef void un_c(self) except *:
    self._trailing_metadata = _metadata(&self._c_trailing_metadata)
    grpc_metadata_array_destroy(&self._c_trailing_metadata)
    self._code = self._c_code
    self._details = _decode(_slice_bytes(self._c_details))
    grpc_slice_unref(self._c_details)
    if self._c_error_string != NULL:
      self._error_string = _decode(self._c_error_string)
      gpr_free(<void*>self._c_error_string)
    else:
      self._error_string = ""

  def trailing_metadata(self):
    return self._trailing_metadata

  def code(self):
    return self._code

  def details(self):
    return self._details

  def error_string(self):
    return self._error_string


cdef class ReceiveCloseOnServerOperation(Operation):

  def __cinit__(self, flags):
    self._flags = flags

  def type(self):
    return GRPC_OP_RECV_CLOSE_ON_SERVER

  cdef void c(self) except *:
    self.c_op.type = GRPC_OP_RECV_CLOSE_ON_SERVER
    self.c_op.flags = self._flags
    self.c_op.data.receive_close_on_server.cancelled = &self._c_cancelled

  cdef void un_c(self) except *:
    self._cancelled = bool(self._c_cancelled)

  def cancelled(self):
    return self._cancelled



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import demo_pb2 as demo__pb2


class GRPCDemoStub:
    """`service` 是用来给gRPC服务定义方法的, 格式固定, 类似于Golang中定义一个接口
    `service` is used to define methods for gRPC services in a fixed format, similar to defining
    an interface in Golang
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SimpleMethod = channel.unary_unary(
                '/demo.GRPCDemo/SimpleMethod',
                request_serializer=demo__pb2.Request.SerializeToString,
                response_deserializer=demo__pb2.Response.FromString,
                )
        self.ClientStreamingMethod = channel.stream_unary(
                '/demo.GRPCDemo/ClientStreamingMethod',
                request_serializer=demo__pb2.Request.SerializeToString,
                response_deserializer=demo__pb2.Response.FromString,
                )
        self.ServerStreamingMethod = channel.unary_stream(
                '/demo.GRPCDemo/ServerStreamingMethod',
                request_serializer=demo__pb2.Request.SerializeToString,
                response_deserializer=demo__pb2.Response.FromString,
                )
        self.BidirectionalStreamingMethod = channel.stream_stream(
                '/demo.GRPCDemo/BidirectionalStreamingMethod',
                request_serializer=demo__pb2.Request.SerializeToString,
                response_deserializer=demo__pb2.Response.FromString,
                )


class GRPCDemoServicer:
    """`service` 是用来给gRPC服务定义方法的, 格式固定, 类似于Golang中定义一个接口
    `service` is used to define methods for gRPC services in a fixed format, similar to defining
    an interface in Golang
    """

    def SimpleMethod(self, request, context):
        """一元模式(在一次调用中, 客户端只能向服务器传输一次请求数据, 服务器也只能返回一次响应)
        unary-unary(In a single call, the client can only send request once, and the server can
        only respond once.)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ClientStreamingMethod(self, request_iterator, context):
        """客户端流模式（在一次调用中, 客户端可以多次向服务器传输数据, 但是服务器只能返回一次响应）
        stream-unary (In a single call, the client can transfer data to the server several times,
        but the server can only return a response once.)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ServerStreamingMethod(self, request, context):
        """服务端流模式（在一次调用中, 客户端只能一次向服务器传输数据, 但是服务器可以多次返回响应）
        unary-stream (In a single call, the client can only transmit data to the server at one time,
        but the server can return the response many times.)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BidirectionalStreamingMethod(self, request_iterator, context):
        """双向流模式 (在一次调用中, 客户端和服务器都可以向对方多次收发数据)
        stream-stream (In a single call, both client and server can send and receive data
        to each other multiple times.)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GRPCDemoServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SimpleMethod': grpc.unary_unary_rpc_method_handler(
                    servicer.SimpleMethod,
                    request_deserializer=demo__pb2.Request.FromString,
                    response_serializer=demo__pb2.Response.SerializeToString,
            ),
            'ClientStreamingMethod': grpc.stream_unary_rpc_method_handler(
                    servicer.ClientStreamingMethod,
                    request_deserializer=demo__pb2.Request.FromString,
                    response_serializer=demo__pb2.Response.SerializeToString,
            ),
            'ServerStreamingMethod': grpc.unary_stream_rpc_method_handler(
                    servicer.ServerStreamingMethod,
                    request_deserializer=demo__pb2.Request.FromString,
                    response_serializer=demo__pb2.Response.SerializeToString,
            ),
            'BidirectionalStreamingMethod': grpc.stream_stream_rpc_method_handler(
                    servicer.BidirectionalStreamingMethod,
                    request_deserializer=demo__pb2.Request.FromString,
                    response_serializer=demo__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'demo.GRPCDemo', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class GRPCDemo:
    """`service` 是用来给gRPC服务定义方法的, 格式固定, 类似于Golang中定义一个接口
    `service` is used to define methods for gRPC services in a fixed format, similar to defining
    an interface in Golang
    """

    @staticmethod
    def SimpleMethod(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/demo.GRPCDemo/SimpleMethod',
            demo__pb2.Request.SerializeToString,
            demo__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ClientStreamingMethod(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/demo.GRPCDemo/ClientStreamingMethod',
            demo__pb2.Request.SerializeToString,
            demo__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ServerStreamingMethod(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/demo.GRPCDemo/ServerStreamingMethod',
            demo__pb2.Request.SerializeToString,
            demo__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BidirectionalStreamingMethod(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/demo.GRPCDemo/BidirectionalStreamingMethod',
            demo__pb2.Request.SerializeToString,
            demo__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#! /usr/bin/env python3
# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Builds the content of xds-protos package"""

import os
import sys

from grpc_tools import protoc

if sys.version_info >= (3, 9, 0):
    from importlib import resources
else:
    import pkg_resources


def localize_path(p):
    return os.path.join(*p.split("/"))


def _get_resource_file_name(
    package_or_requirement: str, resource_name: str
) -> str:
    """Obtain the filename for a resource on the file system."""
    file_name = None
    if sys.version_info >= (3, 9, 0):
        file_name = (
            resources.files(package_or_requirement) / resource_name
        ).resolve()
    else:
        file_name = pkg_resources.resource_filename(
            package_or_requirement, resource_name
        )
    return str(file_name)


# We might not want to compile all the protos
EXCLUDE_PROTO_PACKAGES_LIST = tuple(
    localize_path(p)
    for p in (
        # Requires extra dependency to Prometheus protos
        "envoy/service/metrics/v2",
        "envoy/service/metrics/v3",
        "envoy/service/metrics/v4alpha",
    )
)

# Compute the paths
WORK_DIR = os.path.dirname(os.path.abspath(__file__))
ENVOY_API_PROTO_ROOT = os.path.join("third_party", "envoy-api")
XDS_PROTO_ROOT = os.path.join("third_party", "xds")
GOOGLEAPIS_ROOT = os.path.join("third_party", "googleapis")
CEL_PROTO_ROOT = os.path.join("third_party", "cel-spec", "proto")
VALIDATE_ROOT = os.path.join("third_party", "protoc-gen-validate")
OPENCENSUS_PROTO_ROOT = os.path.join(
    "third_party", "opencensus-proto", "src"
)
OPENTELEMETRY_PROTO_ROOT = os.path.join("third_party", "opentelemetry")
WELL_KNOWN_PROTOS_INCLUDE = _get_resource_file_name("grpc_tools", "_proto")

OUTPUT_PATH = WORK_DIR

# Prepare the test file generation
TEST_FILE_NAME = "generated_file_import_test.py"
TEST_IMPORTS = []

# The pkgutil-style namespace packaging __init__.py
PKGUTIL_STYLE_INIT = (
    "__path__ = __import__('pkgutil').extend_path(__path__, __name__)\n"
)
NAMESPACE_PACKAGES = ["google"]


def add_test_import(proto_package_path: str, file_name: str, service: bool = False):
    TEST_IMPORTS.append(
        "from %s import %s\n"
        % (
            proto_package_path.replace("/", ".").replace("-", "_"),
            file_name.replace(".proto", "_pb2").replace("-", "_"),
        )
    )
    if service:
        TEST_IMPORTS.append(
            "from %s import %s\n"
            % (
                proto_package_path.replace("/", ".").replace("-", "_"),
                file_name.replace(".proto", "_pb2_grpc").replace("-", "_"),
            )
        )


# Prepare Protoc command
COMPILE_PROTO_ONLY = [
    "grpc_tools.protoc",
    "--proto_path={}".format(ENVOY_API_PROTO_ROOT),
    "--proto_path={}".format(XDS_PROTO_ROOT),
    "--proto_path={}".format(GOOGLEAPIS_ROOT),
    "--proto_path={}".format(CEL_PROTO_ROOT),
    "--proto_path={}".format(VALIDATE_ROOT),
    "--proto_path={}".format(WELL_KNOWN_PROTOS_INCLUDE),
    "--proto_path={}".format(OPENCENSUS_PROTO_ROOT),
    "--proto_path={}".format(OPENTELEMETRY_PROTO_ROOT),
    "--python_out={}".format(OUTPUT_PATH),
]
COMPILE_BOTH = COMPILE_PROTO_ONLY + ["--grpc_python_out={}".format(OUTPUT_PATH)]


def has_grpc_service(proto_package_path: str) -> bool:
    return proto_package_path.startswith(os.path.join("envoy", "service"))


def compile_protos(proto_root: str, sub_dir: str = ".") -> None:
    compiled_any = False
    for root, _, files in os.walk(os.path.join(proto_root, sub_dir)):
        proto_package_path = os.path.relpath(root, proto_root)
        if proto_package_path in EXCLUDE_PROTO_PACKAGES_LIST:
            print(f"Skipping package {proto_package_path}")
            continue
        for file_name in files:
            if file_name.endswith(".proto"):
                # Compile proto
                compiled_any = True
                if has_grpc_service(proto_package_path):
                    return_code = protoc.main(
                        COMPILE_BOTH + [os.path.join(root, file_name)]
                    )
                    add_test_import(proto_package_path, file_name, service=True)
                else:
                    return_code = protoc.main(
                        COMPILE_PROTO_ONLY + [os.path.join(root, file_name)]
                    )
                    add_test_import(proto_package_path, file_name, service=False)
                if return_code != 0:
                    raise Exception("error: {} failed".format(COMPILE_BOTH))
    # Ensure a deterministic order.
    TEST_IMPORTS.sort()
    if not compiled_any:
        raise Exception(
            "No proto files found at {}. Did you update git submodules?".format(
                proto_root, sub_dir
            )
        )


def create_init_file(path: str, package_path: str = "") -> None:
    with open(os.path.join(path, "__init__.py"), "w") as f:
        # Apply the pkgutil-style namespace packaging, which is compatible for 2
        # and 3. Here is the full table of namespace compatibility:
        # https://github.com/pypa/sample-namespace-packages/blob/master/table.md
        if package_path in NAMESPACE_PACKAGES:
            f.write(PKGUTIL_STYLE_INIT)


def main():
    # Compile xDS protos
    compile_protos(ENVOY_API_PROTO_ROOT)
    compile_protos(XDS_PROTO_ROOT)
    # We don't want to compile the entire GCP surface API, just the essential ones
    compile_protos(GOOGLEAPIS_ROOT, os.path.join("google", "api"))
    compile_protos(GOOGLEAPIS_ROOT, os.path.join("google", "rpc"))
    compile_protos(GOOGLEAPIS_ROOT, os.path.join("google", "longrunning"))
    compile_protos(GOOGLEAPIS_ROOT, os.path.join("google", "logging"))
    compile_protos(GOOGLEAPIS_ROOT, os.path.join("google", "type"))
    compile_protos(VALIDATE_ROOT, "validate")
    compile_protos(OPENCENSUS_PROTO_ROOT)
    compile_protos(OPENTELEMETRY_PROTO_ROOT)
    compile_protos(CEL_PROTO_ROOT, "cel")

    # Generate __init__.py files for all modules
    create_init_file(WORK_DIR)
    for proto_root_module in [
        "envoy",
        "google",
        "opencensus",
        "udpa",
        "validate",
        "xds",
        "opentelemetry",
        "cel",
    ]:
        for root, _, _ in os.walk(os.path.join(WORK_DIR, proto_root_module)):
            package_path = os.path.relpath(root, WORK_DIR)
            create_init_file(root, package_path)

    # Generate test file
    with open(os.path.join(WORK_DIR, TEST_FILE_NAME), "w") as f:
        f.writelines(TEST_IMPORTS)


if __name__ == "__main__":
    main()



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef class RegisteredMethod:

  def __cinit__(self, bytes method, uintptr_t server):
    self.method = method
    cpython.Py_INCREF(self.method)
    cdef const char *c_method = <const char *>self.method
    cdef grpc_server *c_server = <grpc_server *>server
    # TODO(xuanwn): Consider use GRPC_SRM_PAYLOAD_READ_INITIAL_BYTE_BUFFER for unary request
    # as optimization.
    # Note that in stubs method is not bound to any host, thus we set host as NULL.
    with nogil:
      self.c_registered_method = grpc_server_register_method(c_server,
      c_method, NULL, GRPC_SRM_PAYLOAD_NONE, 0)

  def __dealloc__(self):
    # c_registered_method should have the same lifetime as Cython Server since the method
    # maybe called at any time during the server's lifetime.
    # Since all RegisteredMethod belongs to Cython Server, they'll be destructed at the same
    # time with Cython Server, at which point it's safe to assume core no longer holds any reference.
    cpython.Py_DECREF(self.method)


cdef class Server:

  def __cinit__(self, object arguments, bint xds):
    cdef grpc_server_xds_status_notifier notifier
    cdef _ChannelArgs tmp_channel_args
    cdef grpc_arg fetcher_arg
    cdef grpc_server_config_fetcher* config_fetcher
    cdef _ChannelArgs channel_args
    cdef list arguments_list
    fork_handlers_and_grpc_init()
    self.references = []
    self.registered_completion_queues = []
    self.is_started = False
    self.is_shutting_down = False
    self.is_shutdown = False
    self.c_server = NULL
    self.registered_methods = {}  # Mapping[bytes, RegisteredMethod]
    arguments_list = list(arguments) if arguments is not None else []
    if xds:
      notifier.on_serving_status_update = NULL
      notifier.user_data = NULL
      tmp_channel_args = _ChannelArgs(arguments_list)
      config_fetcher = grpc_server_config_fetcher_xds_create(
          notifier, tmp_channel_args.c_args())
      if config_fetcher != NULL:
        fetcher_arg.key = <char *>GRPC_ARG_SERVER_CONFIG_FETCHER
        fetcher_arg.type = GRPC_ARG_POINTER
        fetcher_arg.value.pointer.vtable = <grpc_arg_pointer_vtable *>grpc_server_config_fetcher_arg_vtable()
        fetcher_arg.value.pointer.address = config_fetcher
        arguments_list.append((GRPC_ARG_SERVER_CONFIG_FETCHER, _wrap_grpc_arg(fetcher_arg)))
    channel_args = _ChannelArgs(arguments_list)
    self.c_server = grpc_server_create(channel_args.c_args(), NULL)
    if xds and config_fetcher != NULL:
      grpc_server_config_fetcher_unref(config_fetcher)
    self.references.append(arguments_list)

  def request_call(
      self, CompletionQueue call_queue not None,
      CompletionQueue server_queue not None, tag):
    if not self.is_started or self.is_shutting_down:
      raise ValueError("server must be started and not shutting down")
    if server_queue not in self.registered_completion_queues:
      raise ValueError("server_queue must be a registered completion queue")
    cdef _RequestCallTag request_call_tag = _RequestCallTag(tag)
    return self._c_request_unregistered_call(request_call_tag, call_queue, server_queue)

  def request_registered_call(
      self, CompletionQueue call_queue not None,
      CompletionQueue server_queue not None,
      str method not None,
      tag):
    if not self.is_started or self.is_shutting_down:
      raise ValueError("server must be started and not shutting down")
    if server_queue not in self.registered_completion_queues:
      raise ValueError("server_queue must be a registered completion queue")
    cdef _RequestCallTag request_call_tag = _RequestCallTag(tag)
    method_bytes = str_to_bytes(method)
    return self._c_request_registered_call(request_call_tag, call_queue, server_queue, method_bytes)

  cdef _c_request_registered_call(self,
       _RequestCallTag request_call_tag,
       CompletionQueue call_queue,
       CompletionQueue server_queue,
       bytes method):
    request_call_tag.prepare()
    cpython.Py_INCREF(request_call_tag)
    cdef cpython.PyObject *c_request_call_tag = <cpython.PyObject *>request_call_tag
    cdef RegisteredMethod registered_method = self.registered_methods[method]
    # optional_payload is set to NULL because we use GRPC_SRM_PAYLOAD_NONE for all method.
    cdef grpc_call_error c_call_error = GRPC_CALL_OK
    with nogil:
      c_call_error = grpc_server_request_registered_call(
          self.c_server, registered_method.c_registered_method, &request_call_tag.call.c_call,
          &request_call_tag.call_details.c_details.deadline,
          &request_call_tag.c_invocation_metadata,
          NULL,
          call_queue.c_completion_queue, server_queue.c_completion_queue,
          c_request_call_tag)
    if c_call_error != GRPC_CALL_OK:
      raise InternalError("Error in grpc_server_request_registered_call: %s" % grpc_call_error_to_string(self.c_call_error).decode())
    return c_call_error

  cdef _c_request_unregistered_call(self,
       _RequestCallTag request_call_tag,
       CompletionQueue call_queue,
       CompletionQueue server_queue):
    request_call_tag.prepare()
    cpython.Py_INCREF(request_call_tag)
    cdef cpython.PyObject *c_request_call_tag = <cpython.PyObject *>request_call_tag
    cdef grpc_call_error c_call_error = GRPC_CALL_OK
    with nogil:
      c_call_error = grpc_server_request_call(
          self.c_server, &request_call_tag.call.c_call,
          &request_call_tag.call_details.c_details,
          &request_call_tag.c_invocation_metadata,
          call_queue.c_completion_queue, server_queue.c_completion_queue,
          c_request_call_tag)
    if c_call_error != GRPC_CALL_OK:
      raise InternalError("Error in grpc_server_request_call: %s" % grpc_call_error_to_string(self.c_call_error).decode())
    return c_call_error

  def register_completion_queue(
      self, CompletionQueue queue not None):
    if self.is_started:
      raise ValueError("cannot register completion queues after start")
    with nogil:
      grpc_server_register_completion_queue(
          self.c_server, queue.c_completion_queue, NULL)
    self.registered_completion_queues.append(queue)

  def register_method(self, str fully_qualified_method):
    method_bytes = str_to_bytes(fully_qualified_method)
    if method_bytes in self.registered_methods.keys():
      # Ignore already registered method
      return
    cdef RegisteredMethod registered_method = RegisteredMethod(method_bytes, <uintptr_t>self.c_server)
    self.registered_methods[method_bytes] = registered_method

  def start(self, backup_queue=True):
    """Start the Cython gRPC Server.
    
    Args:
      backup_queue: a bool indicates whether to spawn a backup completion
        queue. In the case that no CQ is bound to the server, and the shutdown
        of server becomes un-observable.
    """
    if self.is_started:
      raise ValueError("the server has already started")
    if backup_queue:
      self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
      self.register_completion_queue(self.backup_shutdown_queue)
    self.is_started = True
    with nogil:
      grpc_server_start(self.c_server)
    if backup_queue:
      # Ensure the core has gotten a chance to do the start-up work
      self.backup_shutdown_queue.poll(deadline=time.time())

  def add_http2_port(self, bytes address,
                     ServerCredentials server_credentials=None):
    address = str_to_bytes(address)
    self.references.append(address)
    cdef int result
    cdef char *address_c_string = address
    if server_credentials is not None:
      self.references.append(server_credentials)
      with nogil:
        result = grpc_server_add_http2_port(
            self.c_server, address_c_string, server_credentials.c_credentials)
    else:
      with nogil:
        creds = grpc_insecure_server_credentials_create()
        result = grpc_server_add_http2_port(self.c_server,
                                            address_c_string, creds)
        grpc_server_credentials_release(creds)
    return result

  cdef _c_shutdown(self, CompletionQueue queue, tag):
    self.is_shutting_down = True
    cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
    cpython.Py_INCREF(server_shutdown_tag)
    with nogil:
      grpc_server_shutdown_and_notify(
          self.c_server, queue.c_completion_queue,
          <cpython.PyObject *>server_shutdown_tag)

  def shutdown(self, CompletionQueue queue not None, tag):
    if queue.is_shutting_down:
      raise ValueError("queue must be live")
    elif not self.is_started:
      raise ValueError("the server hasn't started yet")
    elif self.is_shutting_down:
      return
    elif queue not in self.registered_completion_queues:
      raise ValueError("expected registered completion queue")
    else:
      self._c_shutdown(queue, tag)

  cdef notify_shutdown_complete(self):
    # called only after our server shutdown tag has emerged from a completion
    # queue.
    self.is_shutdown = True

  def cancel_all_calls(self):
    if not self.is_shutting_down:
      raise UsageError("the server must be shutting down to cancel all calls")
    elif self.is_shutdown:
      return
    else:
      with nogil:
        grpc_server_cancel_all_calls(self.c_server)

  # TODO(https://github.com/grpc/grpc/issues/17515) Determine what, if any,
  # portion of this is safe to call from __dealloc__, and potentially remove
  # backup_shutdown_queue.
  def destroy(self):
    if self.c_server != NULL:
      if not self.is_started:
        pass
      elif self.is_shutdown:
        pass
      elif not self.is_shutting_down:
        if self.backup_shutdown_queue is None:
          raise InternalError('Server shutdown failed: no completion queue.')
        else:
          # the user didn't call shutdown - use our backup queue
          self._c_shutdown(self.backup_shutdown_queue, None)
          # and now we wait
          while not self.is_shutdown:
            self.backup_shutdown_queue.poll()
      else:
        # We're in the process of shutting down, but have not shutdown; can't do
        # much but repeatedly release the GIL and wait
        while not self.is_shutdown:
          time.sleep(0)
      with nogil:
        grpc_server_destroy(self.c_server)
        self.c_server = NULL

  def __dealloc__(self):
    if self.c_server == NULL:
      grpc_shutdown()



# Copyright 2024 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Example gRPC client to depict flow control in gRPC"""

import logging

import grpc
import helloworld_pb2
import helloworld_pb2_grpc
import helpers


def get_iter_data():
    max_iter = 100
    data_size = 2000
    test_request_data = bytes("1" * data_size, "utf-8")

    for i in range(1, (max_iter + 1)):
        if (i % 10) == 0:
            print(
                f"\n{helpers.get_current_time()}   "
                f"Request {i}: Sent {(data_size*i)} bytes in total"
            )

        yield helloworld_pb2.HelloRequest(name=test_request_data)


def run():
    with grpc.insecure_channel(
        "localhost:50051",
        # Setting server options to minimal, to allow low number of maximum
        # bytes in the window
        # `bdp_probe` is set to 0(false) to disable resizing of window
        options=[
            ("grpc.http2.max_frame_size", 16384),
            ("grpc.http2.bdp_probe", 0),
        ],
    ) as channel:
        stub = helloworld_pb2_grpc.GreeterStub(channel)

        responses = stub.SayHelloBidiStream(get_iter_data())
        for i, _ in enumerate(responses, start=1):
            if (i % 10) == 0:
                print(
                    f"{helpers.get_current_time()}   "
                    f"Received {i} responses\n"
                )


if __name__ == "__main__":
    logging.basicConfig()
    run()



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import contextlib
import logging
import os
import subprocess
import sys
import tempfile
import time
from typing import Iterable, List, Mapping, Set, Tuple
import unittest

import grpc.experimental
import xds_interop_client
import xds_interop_server

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2
from src.proto.grpc.testing import test_pb2_grpc
import src.python.grpcio_tests.tests.unit.framework.common as framework_common

_CLIENT_PATH = os.path.abspath(os.path.realpath(xds_interop_client.__file__))
_SERVER_PATH = os.path.abspath(os.path.realpath(xds_interop_server.__file__))

_METHODS = (
    (messages_pb2.ClientConfigureRequest.UNARY_CALL, "UNARY_CALL"),
    (messages_pb2.ClientConfigureRequest.EMPTY_CALL, "EMPTY_CALL"),
)

_QPS = 100
_NUM_CHANNELS = 20

_TEST_ITERATIONS = 10
_ITERATION_DURATION_SECONDS = 1
_SUBPROCESS_TIMEOUT_SECONDS = 2


def _set_union(a: Iterable, b: Iterable) -> Set:
    c = set(a)
    c.update(b)
    return c


@contextlib.contextmanager
def _start_python_with_args(
    file: str, args: List[str]
) -> Tuple[subprocess.Popen, tempfile.TemporaryFile, tempfile.TemporaryFile]:
    with tempfile.TemporaryFile(mode="r") as stdout:
        with tempfile.TemporaryFile(mode="r") as stderr:
            proc = subprocess.Popen(
                (sys.executable, file) + tuple(args),
                stdout=stdout,
                stderr=stderr,
            )
            yield proc, stdout, stderr


def _dump_stream(
    process_name: str, stream_name: str, stream: tempfile.TemporaryFile
):
    sys.stderr.write(f"{process_name} {stream_name}:\n")
    stream.seek(0)
    sys.stderr.write(stream.read())


def _dump_streams(
    process_name: str,
    stdout: tempfile.TemporaryFile,
    stderr: tempfile.TemporaryFile,
):
    _dump_stream(process_name, "stdout", stdout)
    _dump_stream(process_name, "stderr", stderr)
    sys.stderr.write(f"End {process_name} output.\n")


def _index_accumulated_stats(
    response: messages_pb2.LoadBalancerAccumulatedStatsResponse,
) -> Mapping[str, Mapping[int, int]]:
    indexed = collections.defaultdict(lambda: collections.defaultdict(int))
    for _, method_str in _METHODS:
        for status in response.stats_per_method[method_str].result.keys():
            indexed[method_str][status] = response.stats_per_method[
                method_str
            ].result[status]
    return indexed


def _subtract_indexed_stats(
    a: Mapping[str, Mapping[int, int]], b: Mapping[str, Mapping[int, int]]
):
    c = collections.defaultdict(lambda: collections.defaultdict(int))
    all_methods = _set_union(a.keys(), b.keys())
    for method in all_methods:
        all_statuses = _set_union(a[method].keys(), b[method].keys())
        for status in all_statuses:
            c[method][status] = a[method][status] - b[method][status]
    return c


def _collect_stats(
    stats_port: int, duration: int
) -> Mapping[str, Mapping[int, int]]:
    settings = {
        "target": f"localhost:{stats_port}",
        "insecure": True,
    }
    response = test_pb2_grpc.LoadBalancerStatsService.GetClientAccumulatedStats(
        messages_pb2.LoadBalancerAccumulatedStatsRequest(), **settings
    )
    before = _index_accumulated_stats(response)
    time.sleep(duration)
    response = test_pb2_grpc.LoadBalancerStatsService.GetClientAccumulatedStats(
        messages_pb2.LoadBalancerAccumulatedStatsRequest(), **settings
    )
    after = _index_accumulated_stats(response)
    return _subtract_indexed_stats(after, before)


class XdsInteropClientTest(unittest.TestCase):
    def _assert_client_consistent(
        self, server_port: int, stats_port: int, qps: int, num_channels: int
    ):
        settings = {
            "target": f"localhost:{stats_port}",
            "insecure": True,
        }
        for i in range(_TEST_ITERATIONS):
            target_method, target_method_str = _METHODS[i % len(_METHODS)]
            test_pb2_grpc.XdsUpdateClientConfigureService.Configure(
                messages_pb2.ClientConfigureRequest(types=[target_method]),
                **settings,
            )
            delta = _collect_stats(stats_port, _ITERATION_DURATION_SECONDS)
            logging.info("Delta: %s", delta)
            for _, method_str in _METHODS:
                for status in delta[method_str]:
                    if status == 0 and method_str == target_method_str:
                        self.assertGreater(delta[method_str][status], 0, delta)
                    else:
                        self.assertEqual(delta[method_str][status], 0, delta)

    def test_configure_consistency(self):
        _, server_port, socket = framework_common.get_socket()

        with _start_python_with_args(
            _SERVER_PATH,
            [f"--port={server_port}", f"--maintenance_port={server_port}"],
        ) as (server, server_stdout, server_stderr):
            # Send RPC to server to make sure it's running.
            logging.info("Sending RPC to server.")
            test_pb2_grpc.TestService.EmptyCall(
                empty_pb2.Empty(),
                f"localhost:{server_port}",
                insecure=True,
                wait_for_ready=True,
            )
            logging.info("Server successfully started.")
            socket.close()
            _, stats_port, stats_socket = framework_common.get_socket()
            with _start_python_with_args(
                _CLIENT_PATH,
                [
                    f"--server=localhost:{server_port}",
                    f"--stats_port={stats_port}",
                    f"--qps={_QPS}",
                    f"--num_channels={_NUM_CHANNELS}",
                ],
            ) as (client, client_stdout, client_stderr):
                stats_socket.close()
                try:
                    self._assert_client_consistent(
                        server_port, stats_port, _QPS, _NUM_CHANNELS
                    )
                except:
                    _dump_streams("server", server_stdout, server_stderr)
                    _dump_streams("client", client_stdout, client_stderr)
                    raise
                finally:
                    server.kill()
                    client.kill()
                    server.wait(timeout=_SUBPROCESS_TIMEOUT_SECONDS)
                    client.wait(timeout=_SUBPROCESS_TIMEOUT_SECONDS)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/tap/v3/common.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/tap/v3/common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.common.matcher.v3 import matcher_pb2 as envoy_dot_config_dot_common_dot_matcher_dot_v3_dot_matcher__pb2
from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from envoy.config.core.v3 import grpc_service_pb2 as envoy_dot_config_dot_core_dot_v3_dot_grpc__service__pb2
from envoy.config.route.v3 import route_components_pb2 as envoy_dot_config_dot_route_dot_v3_dot_route__components__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n envoy/config/tap/v3/common.proto\x12\x13\x65nvoy.config.tap.v3\x1a,envoy/config/common/matcher/v3/matcher.proto\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a$envoy/config/core/v3/extension.proto\x1a\'envoy/config/core/v3/grpc_service.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xc7\x02\n\tTapConfig\x12\x46\n\x0cmatch_config\x18\x01 \x01(\x0b\x32#.envoy.config.tap.v3.MatchPredicateB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12=\n\x05match\x18\x04 \x01(\x0b\x32..envoy.config.common.matcher.v3.MatchPredicate\x12\x42\n\routput_config\x18\x02 \x01(\x0b\x32!.envoy.config.tap.v3.OutputConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x43\n\x0btap_enabled\x18\x03 \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercent:*\x9a\xc5\x88\x1e%\n#envoy.service.tap.v2alpha.TapConfig\"\x95\x07\n\x0eMatchPredicate\x12@\n\x08or_match\x18\x01 \x01(\x0b\x32,.envoy.config.tap.v3.MatchPredicate.MatchSetH\x00\x12\x41\n\tand_match\x18\x02 \x01(\x0b\x32,.envoy.config.tap.v3.MatchPredicate.MatchSetH\x00\x12\x38\n\tnot_match\x18\x03 \x01(\x0b\x32#.envoy.config.tap.v3.MatchPredicateH\x00\x12\x1c\n\tany_match\x18\x04 \x01(\x08\x42\x07\xfa\x42\x04j\x02\x08\x01H\x00\x12K\n\x1ahttp_request_headers_match\x18\x05 \x01(\x0b\x32%.envoy.config.tap.v3.HttpHeadersMatchH\x00\x12L\n\x1bhttp_request_trailers_match\x18\x06 \x01(\x0b\x32%.envoy.config.tap.v3.HttpHeadersMatchH\x00\x12L\n\x1bhttp_response_headers_match\x18\x07 \x01(\x0b\x32%.envoy.config.tap.v3.HttpHeadersMatchH\x00\x12M\n\x1chttp_response_trailers_match\x18\x08 \x01(\x0b\x32%.envoy.config.tap.v3.HttpHeadersMatchH\x00\x12T\n\x1fhttp_request_generic_body_match\x18\t \x01(\x0b\x32).envoy.config.tap.v3.HttpGenericBodyMatchH\x00\x12U\n http_response_generic_body_match\x18\n \x01(\x0b\x32).envoy.config.tap.v3.HttpGenericBodyMatchH\x00\x1a\x82\x01\n\x08MatchSet\x12<\n\x05rules\x18\x01 \x03(\x0b\x32#.envoy.config.tap.v3.MatchPredicateB\x08\xfa\x42\x05\x92\x01\x02\x08\x02:8\x9a\xc5\x88\x1e\x33\n1envoy.service.tap.v2alpha.MatchPredicate.MatchSet:/\x9a\xc5\x88\x1e*\n(envoy.service.tap.v2alpha.MatchPredicateB\x0b\n\x04rule\x12\x03\xf8\x42\x01\"|\n\x10HttpHeadersMatch\x12\x35\n\x07headers\x18\x01 \x03(\x0b\x32$.envoy.config.route.v3.HeaderMatcher:1\x9a\xc5\x88\x1e,\n*envoy.service.tap.v2alpha.HttpHeadersMatch\"\xe6\x01\n\x14HttpGenericBodyMatch\x12\x13\n\x0b\x62ytes_limit\x18\x01 \x01(\r\x12V\n\x08patterns\x18\x02 \x03(\x0b\x32:.envoy.config.tap.v3.HttpGenericBodyMatch.GenericTextMatchB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x61\n\x10GenericTextMatch\x12\x1f\n\x0cstring_match\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x12\x1f\n\x0c\x62inary_match\x18\x02 \x01(\x0c\x42\x07\xfa\x42\x04z\x02\x10\x01H\x00\x42\x0b\n\x04rule\x12\x03\xf8\x42\x01\"\xc5\x02\n\x0cOutputConfig\x12:\n\x05sinks\x18\x01 \x03(\x0b\x32\x1f.envoy.config.tap.v3.OutputSinkB\n\xfa\x42\x07\x92\x01\x04\x08\x01\x10\x01\x12;\n\x15max_buffered_rx_bytes\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12;\n\x15max_buffered_tx_bytes\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x11\n\tstreaming\x18\x04 \x01(\x08\x12=\n\x17min_streamed_sent_bytes\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value:-\x9a\xc5\x88\x1e(\n&envoy.service.tap.v2alpha.OutputConfig\"\xdc\x04\n\nOutputSink\x12@\n\x06\x66ormat\x18\x01 \x01(\x0e\x32&.envoy.config.tap.v3.OutputSink.FormatB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x42\n\x0fstreaming_admin\x18\x02 \x01(\x0b\x32\'.envoy.config.tap.v3.StreamingAdminSinkH\x00\x12;\n\x0c\x66ile_per_tap\x18\x03 \x01(\x0b\x32#.envoy.config.tap.v3.FilePerTapSinkH\x00\x12@\n\x0estreaming_grpc\x18\x04 \x01(\x0b\x32&.envoy.config.tap.v3.StreamingGrpcSinkH\x00\x12@\n\x0e\x62uffered_admin\x18\x05 \x01(\x0b\x32&.envoy.config.tap.v3.BufferedAdminSinkH\x00\x12\x41\n\x0b\x63ustom_sink\x18\x06 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfigH\x00\"~\n\x06\x46ormat\x12\x16\n\x12JSON_BODY_AS_BYTES\x10\x00\x12\x17\n\x13JSON_BODY_AS_STRING\x10\x01\x12\x10\n\x0cPROTO_BINARY\x10\x02\x12!\n\x1dPROTO_BINARY_LENGTH_DELIMITED\x10\x03\x12\x0e\n\nPROTO_TEXT\x10\x04:+\x9a\xc5\x88\x1e&\n$envoy.service.tap.v2alpha.OutputSinkB\x17\n\x10output_sink_type\x12\x03\xf8\x42\x01\"I\n\x12StreamingAdminSink:3\x9a\xc5\x88\x1e.\n,envoy.service.tap.v2alpha.StreamingAdminSink\"\\\n\x11\x42ufferedAdminSink\x12\x1b\n\nmax_traces\x18\x01 \x01(\x04\x42\x07\xfa\x42\x04\x32\x02 \x00\x12*\n\x07timeout\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\"_\n\x0e\x46ilePerTapSink\x12\x1c\n\x0bpath_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:/\x9a\xc5\x88\x1e*\n(envoy.service.tap.v2alpha.FilePerTapSink\"\x9a\x01\n\x11StreamingGrpcSink\x12\x0e\n\x06tap_id\x18\x01 \x01(\t\x12\x41\n\x0cgrpc_service\x18\x02 \x01(\x0b\x32!.envoy.config.core.v3.GrpcServiceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01:2\x9a\xc5\x88\x1e-\n+envoy.service.tap.v2alpha.StreamingGrpcSinkB|\n!io.envoyproxy.envoy.config.tap.v3B\x0b\x43ommonProtoP\x01Z@github.com/envoyproxy/go-control-plane/envoy/config/tap/v3;tapv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.tap.v3.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!io.envoyproxy.envoy.config.tap.v3B\013CommonProtoP\001Z@github.com/envoyproxy/go-control-plane/envoy/config/tap/v3;tapv3\272\200\310\321\006\002\020\002'
  _globals['_TAPCONFIG'].fields_by_name['match_config']._loaded_options = None
  _globals['_TAPCONFIG'].fields_by_name['match_config']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_TAPCONFIG'].fields_by_name['output_config']._loaded_options = None
  _globals['_TAPCONFIG'].fields_by_name['output_config']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_TAPCONFIG']._loaded_options = None
  _globals['_TAPCONFIG']._serialized_options = b'\232\305\210\036%\n#envoy.service.tap.v2alpha.TapConfig'
  _globals['_MATCHPREDICATE_MATCHSET'].fields_by_name['rules']._loaded_options = None
  _globals['_MATCHPREDICATE_MATCHSET'].fields_by_name['rules']._serialized_options = b'\372B\005\222\001\002\010\002'
  _globals['_MATCHPREDICATE_MATCHSET']._loaded_options = None
  _globals['_MATCHPREDICATE_MATCHSET']._serialized_options = b'\232\305\210\0363\n1envoy.service.tap.v2alpha.MatchPredicate.MatchSet'
  _globals['_MATCHPREDICATE'].oneofs_by_name['rule']._loaded_options = None
  _globals['_MATCHPREDICATE'].oneofs_by_name['rule']._serialized_options = b'\370B\001'
  _globals['_MATCHPREDICATE'].fields_by_name['any_match']._loaded_options = None
  _globals['_MATCHPREDICATE'].fields_by_name['any_match']._serialized_options = b'\372B\004j\002\010\001'
  _globals['_MATCHPREDICATE']._loaded_options = None
  _globals['_MATCHPREDICATE']._serialized_options = b'\232\305\210\036*\n(envoy.service.tap.v2alpha.MatchPredicate'
  _globals['_HTTPHEADERSMATCH']._loaded_options = None
  _globals['_HTTPHEADERSMATCH']._serialized_options = b'\232\305\210\036,\n*envoy.service.tap.v2alpha.HttpHeadersMatch'
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH'].oneofs_by_name['rule']._loaded_options = None
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH'].oneofs_by_name['rule']._serialized_options = b'\370B\001'
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH'].fields_by_name['string_match']._loaded_options = None
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH'].fields_by_name['string_match']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH'].fields_by_name['binary_match']._loaded_options = None
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH'].fields_by_name['binary_match']._serialized_options = b'\372B\004z\002\020\001'
  _globals['_HTTPGENERICBODYMATCH'].fields_by_name['patterns']._loaded_options = None
  _globals['_HTTPGENERICBODYMATCH'].fields_by_name['patterns']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_OUTPUTCONFIG'].fields_by_name['sinks']._loaded_options = None
  _globals['_OUTPUTCONFIG'].fields_by_name['sinks']._serialized_options = b'\372B\007\222\001\004\010\001\020\001'
  _globals['_OUTPUTCONFIG']._loaded_options = None
  _globals['_OUTPUTCONFIG']._serialized_options = b'\232\305\210\036(\n&envoy.service.tap.v2alpha.OutputConfig'
  _globals['_OUTPUTSINK'].oneofs_by_name['output_sink_type']._loaded_options = None
  _globals['_OUTPUTSINK'].oneofs_by_name['output_sink_type']._serialized_options = b'\370B\001'
  _globals['_OUTPUTSINK'].fields_by_name['format']._loaded_options = None
  _globals['_OUTPUTSINK'].fields_by_name['format']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_OUTPUTSINK']._loaded_options = None
  _globals['_OUTPUTSINK']._serialized_options = b'\232\305\210\036&\n$envoy.service.tap.v2alpha.OutputSink'
  _globals['_STREAMINGADMINSINK']._loaded_options = None
  _globals['_STREAMINGADMINSINK']._serialized_options = b'\232\305\210\036.\n,envoy.service.tap.v2alpha.StreamingAdminSink'
  _globals['_BUFFEREDADMINSINK'].fields_by_name['max_traces']._loaded_options = None
  _globals['_BUFFEREDADMINSINK'].fields_by_name['max_traces']._serialized_options = b'\372B\0042\002 \000'
  _globals['_FILEPERTAPSINK'].fields_by_name['path_prefix']._loaded_options = None
  _globals['_FILEPERTAPSINK'].fields_by_name['path_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FILEPERTAPSINK']._loaded_options = None
  _globals['_FILEPERTAPSINK']._serialized_options = b'\232\305\210\036*\n(envoy.service.tap.v2alpha.FilePerTapSink'
  _globals['_STREAMINGGRPCSINK'].fields_by_name['grpc_service']._loaded_options = None
  _globals['_STREAMINGGRPCSINK'].fields_by_name['grpc_service']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_STREAMINGGRPCSINK']._loaded_options = None
  _globals['_STREAMINGGRPCSINK']._serialized_options = b'\232\305\210\036-\n+envoy.service.tap.v2alpha.StreamingGrpcSink'
  _globals['_TAPCONFIG']._serialized_start=454
  _globals['_TAPCONFIG']._serialized_end=781
  _globals['_MATCHPREDICATE']._serialized_start=784
  _globals['_MATCHPREDICATE']._serialized_end=1701
  _globals['_MATCHPREDICATE_MATCHSET']._serialized_start=1509
  _globals['_MATCHPREDICATE_MATCHSET']._serialized_end=1639
  _globals['_HTTPHEADERSMATCH']._serialized_start=1703
  _globals['_HTTPHEADERSMATCH']._serialized_end=1827
  _globals['_HTTPGENERICBODYMATCH']._serialized_start=1830
  _globals['_HTTPGENERICBODYMATCH']._serialized_end=2060
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH']._serialized_start=1963
  _globals['_HTTPGENERICBODYMATCH_GENERICTEXTMATCH']._serialized_end=2060
  _globals['_OUTPUTCONFIG']._serialized_start=2063
  _globals['_OUTPUTCONFIG']._serialized_end=2388
  _globals['_OUTPUTSINK']._serialized_start=2391
  _globals['_OUTPUTSINK']._serialized_end=2995
  _globals['_OUTPUTSINK_FORMAT']._serialized_start=2799
  _globals['_OUTPUTSINK_FORMAT']._serialized_end=2925
  _globals['_STREAMINGADMINSINK']._serialized_start=2997
  _globals['_STREAMINGADMINSINK']._serialized_end=3070
  _globals['_BUFFEREDADMINSINK']._serialized_start=3072
  _globals['_BUFFEREDADMINSINK']._serialized_end=3164
  _globals['_FILEPERTAPSINK']._serialized_start=3166
  _globals['_FILEPERTAPSINK']._serialized_end=3261
  _globals['_STREAMINGGRPCSINK']._serialized_start=3264
  _globals['_STREAMINGGRPCSINK']._serialized_end=3418
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests behavior of the timeout mechanism on client side."""

import asyncio
import datetime
import logging
import platform
import random
import unittest

import grpc
from grpc.experimental import aio

from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase

_SLEEP_TIME_UNIT_S = datetime.timedelta(seconds=1).total_seconds()

_TEST_SLEEPY_UNARY_UNARY = "/test/Test/SleepyUnaryUnary"
_TEST_SLEEPY_UNARY_STREAM = "/test/Test/SleepyUnaryStream"
_TEST_SLEEPY_STREAM_UNARY = "/test/Test/SleepyStreamUnary"
_TEST_SLEEPY_STREAM_STREAM = "/test/Test/SleepyStreamStream"

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x01\x01\x01"


async def _test_sleepy_unary_unary(unused_request, unused_context):
    await asyncio.sleep(_SLEEP_TIME_UNIT_S)
    return _RESPONSE


async def _test_sleepy_unary_stream(unused_request, unused_context):
    yield _RESPONSE
    await asyncio.sleep(_SLEEP_TIME_UNIT_S)
    yield _RESPONSE


async def _test_sleepy_stream_unary(unused_request_iterator, context):
    assert _REQUEST == await context.read()
    await asyncio.sleep(_SLEEP_TIME_UNIT_S)
    assert _REQUEST == await context.read()
    return _RESPONSE


async def _test_sleepy_stream_stream(unused_request_iterator, context):
    assert _REQUEST == await context.read()
    await asyncio.sleep(_SLEEP_TIME_UNIT_S)
    await context.write(_RESPONSE)


_ROUTING_TABLE = {
    _TEST_SLEEPY_UNARY_UNARY: grpc.unary_unary_rpc_method_handler(
        _test_sleepy_unary_unary
    ),
    _TEST_SLEEPY_UNARY_STREAM: grpc.unary_stream_rpc_method_handler(
        _test_sleepy_unary_stream
    ),
    _TEST_SLEEPY_STREAM_UNARY: grpc.stream_unary_rpc_method_handler(
        _test_sleepy_stream_unary
    ),
    _TEST_SLEEPY_STREAM_STREAM: grpc.stream_stream_rpc_method_handler(
        _test_sleepy_stream_stream
    ),
}


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        return _ROUTING_TABLE.get(handler_call_details.method)


async def _start_test_server():
    server = aio.server()
    port = server.add_insecure_port("[::]:0")
    server.add_generic_rpc_handlers((_GenericHandler(),))
    await server.start()
    return f"localhost:{port}", server


class TestTimeout(AioTestBase):
    async def setUp(self):
        address, self._server = await _start_test_server()
        self._client = aio.insecure_channel(address)
        self.assertEqual(
            grpc.ChannelConnectivity.IDLE, self._client.get_state(True)
        )
        await _common.block_until_certain_state(
            self._client, grpc.ChannelConnectivity.READY
        )

    async def tearDown(self):
        await self._client.close()
        await self._server.stop(None)

    async def test_unary_unary_success_with_timeout(self):
        multicallable = self._client.unary_unary(_TEST_SLEEPY_UNARY_UNARY)
        call = multicallable(_REQUEST, timeout=2 * _SLEEP_TIME_UNIT_S)
        self.assertEqual(_RESPONSE, await call)
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_unary_unary_deadline_exceeded(self):
        multicallable = self._client.unary_unary(_TEST_SLEEPY_UNARY_UNARY)
        call = multicallable(_REQUEST, timeout=0.5 * _SLEEP_TIME_UNIT_S)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call

        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.DEADLINE_EXCEEDED, rpc_error.code())

    async def test_unary_stream_success_with_timeout(self):
        multicallable = self._client.unary_stream(_TEST_SLEEPY_UNARY_STREAM)
        call = multicallable(_REQUEST, timeout=2 * _SLEEP_TIME_UNIT_S)
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_unary_stream_deadline_exceeded(self):
        multicallable = self._client.unary_stream(_TEST_SLEEPY_UNARY_STREAM)
        call = multicallable(_REQUEST, timeout=0.5 * _SLEEP_TIME_UNIT_S)
        self.assertEqual(_RESPONSE, await call.read())

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.read()

        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.DEADLINE_EXCEEDED, rpc_error.code())

    async def test_stream_unary_success_with_timeout(self):
        multicallable = self._client.stream_unary(_TEST_SLEEPY_STREAM_UNARY)
        call = multicallable(timeout=2 * _SLEEP_TIME_UNIT_S)
        await call.write(_REQUEST)
        await call.write(_REQUEST)
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_stream_unary_deadline_exceeded(self):
        multicallable = self._client.stream_unary(_TEST_SLEEPY_STREAM_UNARY)
        call = multicallable(timeout=0.5 * _SLEEP_TIME_UNIT_S)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.write(_REQUEST)
            await call.write(_REQUEST)
            await call

        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.DEADLINE_EXCEEDED, rpc_error.code())

    async def test_stream_stream_success_with_timeout(self):
        multicallable = self._client.stream_stream(_TEST_SLEEPY_STREAM_STREAM)
        call = multicallable(timeout=2 * _SLEEP_TIME_UNIT_S)
        await call.write(_REQUEST)
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_stream_stream_deadline_exceeded(self):
        multicallable = self._client.stream_stream(_TEST_SLEEPY_STREAM_STREAM)
        call = multicallable(timeout=0.5 * _SLEEP_TIME_UNIT_S)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.write(_REQUEST)
            await call.read()

        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.DEADLINE_EXCEEDED, rpc_error.code())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random
import threading
import time
import unittest

import grpc_testing

_QUANTUM = 0.3
_MANY = 10000
# Tests that run in real time can either wait for the scheduler to
# eventually run what needs to be run (and risk timing out) or declare
# that the scheduler didn't schedule work reasonably fast enough. We
# choose the latter for this test.
_PATHOLOGICAL_SCHEDULING = "pathological thread scheduling!"


class _TimeNoter:
    def __init__(self, time):
        self._condition = threading.Condition()
        self._time = time
        self._call_times = []

    def __call__(self):
        with self._condition:
            self._call_times.append(self._time.time())

    def call_times(self):
        with self._condition:
            return tuple(self._call_times)


class TimeTest:
    def test_sleep_for(self):
        start_time = self._time.time()
        self._time.sleep_for(_QUANTUM)
        end_time = self._time.time()

        self.assertLessEqual(start_time + _QUANTUM, end_time)

    def test_sleep_until(self):
        start_time = self._time.time()
        self._time.sleep_until(start_time + _QUANTUM)
        end_time = self._time.time()

        self.assertLessEqual(start_time + _QUANTUM, end_time)

    def test_call_in(self):
        time_noter = _TimeNoter(self._time)

        start_time = self._time.time()
        self._time.call_in(time_noter, _QUANTUM)
        self._time.sleep_for(_QUANTUM * 2)
        call_times = time_noter.call_times()

        self.assertTrue(call_times, msg=_PATHOLOGICAL_SCHEDULING)
        self.assertLessEqual(start_time + _QUANTUM, call_times[0])

    def test_call_at(self):
        time_noter = _TimeNoter(self._time)

        start_time = self._time.time()
        self._time.call_at(time_noter, self._time.time() + _QUANTUM)
        self._time.sleep_for(_QUANTUM * 2)
        call_times = time_noter.call_times()

        self.assertTrue(call_times, msg=_PATHOLOGICAL_SCHEDULING)
        self.assertLessEqual(start_time + _QUANTUM, call_times[0])

    def test_cancel(self):
        time_noter = _TimeNoter(self._time)

        future = self._time.call_in(time_noter, _QUANTUM * 2)
        self._time.sleep_for(_QUANTUM)
        cancelled = future.cancel()
        self._time.sleep_for(_QUANTUM * 2)
        call_times = time_noter.call_times()

        self.assertFalse(call_times, msg=_PATHOLOGICAL_SCHEDULING)
        self.assertTrue(cancelled)
        self.assertTrue(future.cancelled())

    def test_many(self):
        test_events = tuple(threading.Event() for _ in range(_MANY))
        possibly_cancelled_futures = {}
        background_noise_futures = []

        for test_event in test_events:
            possibly_cancelled_futures[test_event] = self._time.call_in(
                test_event.set, _QUANTUM * (2 + random.random())
            )
        for _ in range(_MANY):
            background_noise_futures.append(
                self._time.call_in(
                    threading.Event().set, _QUANTUM * 1000 * random.random()
                )
            )
        self._time.sleep_for(_QUANTUM)
        cancelled = set()
        for test_event, test_future in possibly_cancelled_futures.items():
            if bool(random.randint(0, 1)) and test_future.cancel():
                cancelled.add(test_event)
        self._time.sleep_for(_QUANTUM * 3)

        for test_event in test_events:
            (self.assertFalse if test_event in cancelled else self.assertTrue)(
                test_event.is_set()
            )
        for background_noise_future in background_noise_futures:
            background_noise_future.cancel()

    def test_same_behavior_used_several_times(self):
        time_noter = _TimeNoter(self._time)

        start_time = self._time.time()
        first_future_at_one = self._time.call_in(time_noter, _QUANTUM)
        second_future_at_one = self._time.call_in(time_noter, _QUANTUM)
        first_future_at_three = self._time.call_in(time_noter, _QUANTUM * 3)
        second_future_at_three = self._time.call_in(time_noter, _QUANTUM * 3)
        self._time.sleep_for(_QUANTUM * 2)
        first_future_at_one_cancelled = first_future_at_one.cancel()
        second_future_at_one_cancelled = second_future_at_one.cancel()
        first_future_at_three_cancelled = first_future_at_three.cancel()
        self._time.sleep_for(_QUANTUM * 2)
        second_future_at_three_cancelled = second_future_at_three.cancel()
        first_future_at_three_cancelled_again = first_future_at_three.cancel()
        call_times = time_noter.call_times()

        self.assertEqual(3, len(call_times), msg=_PATHOLOGICAL_SCHEDULING)
        self.assertFalse(first_future_at_one_cancelled)
        self.assertFalse(second_future_at_one_cancelled)
        self.assertTrue(first_future_at_three_cancelled)
        self.assertFalse(second_future_at_three_cancelled)
        self.assertTrue(first_future_at_three_cancelled_again)
        self.assertLessEqual(start_time + _QUANTUM, call_times[0])
        self.assertLessEqual(start_time + _QUANTUM, call_times[1])
        self.assertLessEqual(start_time + _QUANTUM * 3, call_times[2])


class StrictRealTimeTest(TimeTest, unittest.TestCase):
    def setUp(self):
        self._time = grpc_testing.strict_real_time()


class StrictFakeTimeTest(TimeTest, unittest.TestCase):
    def setUp(self):
        self._time = grpc_testing.strict_fake_time(
            random.randint(0, int(time.time()))
        )


if __name__ == "__main__":
    unittest.main(verbosity=2)
